#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class Snapshot(Resource):
    """Snapshots are point-in-time copies of a volume. Snapshots are managed the same way you manage volumes. In reality, snapshots are volumes: they can be accessed by initiators,
    are subject to the same controls, can be modified, and have the same restrictions as volumes. Snapshots can be cloned and replicated. The initial snapshot uses no space: it
    shares the original data with the source volume. Each successive snapshot captures the changes that have occurred on the volume. The changed blocks are compressed.

    # Parameters
    id                          : Identifier for the snapshot.
    name                        : Name of snapshot.
    description                 : Text description of snapshot.
    size                        : Size of volume at time of snapshot (in bytes).
    vol_name                    : Name of the parent volume in which the snapshot belongs to.
    pool_name                   : Name of the pool in which the parent volume belongs to.
    vol_id                      : Parent volume ID.
    snap_collection_name        : Name of snapshot collection.
    snap_collection_id          : Identifier of snapshot collection.
    online                      : Online state for a snapshot means it could be mounted for data restore.
    writable                    : Allow snapshot to be writable. Mandatory and must be set to 'true' for VSS application synchronized snapshots.
    offline_reason              : Snapshot offline reason - possible entries: one of 'user', 'recovery', 'replica', 'over_volume_limit', 'over_snapshot_limit',
                                  'over_volume_reserve', 'nvram_loss_recovery', 'pool_free_space_exhausted' .
    expiry_time                 : Unix timestamp indicating that the snapshot is considered expired by Snapshot Time-to-live(TTL). A value of 0 indicates that snapshot never
                                  expires.
    expiry_after                : Number of seconds after which this snapshot is considered expired by snapshot TTL. A value of 0 indicates that snapshot never expires, 1
                                  indicates that snapshot uses group-level configured TTL value and any other value indicates number of seconds.
    origin_name                 : Origination group name.
    is_replica                  : Snapshot is a replica from upstream replication partner.
    is_unmanaged                : Indicates whether the snapshot is unmanaged. The snapshot will not be deleted automatically unless the unmanaged cleanup feature is enabled.
    is_manually_managed         : Is snapshot manually managed, i.e., snapshot is manually or third party created or created by system at the time of volume restore or resize.
    replication_status          : Replication status.
    access_control_records      : List of access control records that apply to this snapshot.
    serial_number               : Identifier for the SCSI protocol.
    target_name                 : The iSCSI Qualified Name (IQN) or the Fibre Channel World Wide Node Name (WWNN) of the target snapshot.
    creation_time               : Time when this snapshot was created.
    last_modified               : Time when this snapshort was last modified.
    schedule_name               : Name of protection schedule.
    schedule_id                 : Identifier of protection schedule.
    app_uuid                    : Application identifier of snapshots.
    metadata                    : Key-value pairs that augment a snapshot's attributes.
    new_data_valid              : Indicate the usage infomation is valid.
    new_data_compressed_bytes   : The bytes of compressed new data.
    new_data_uncompressed_bytes : The bytes of uncompressed new data.
    agent_type                  : External management agent type.
    vpd_t10                     : The snapshot's T10 Vendor ID-based identifier.
    vpd_ieee0                   : The first 64 bits of the snapshots's EUI-64 identifier, encoded as a hexadecimal string.
    vpd_ieee1                   : The last 64 bits of the snapshots's EUI-64 identifier, encoded as a hexadecimal string.
    force                       : Forcibly delete the specified snapshot even if it is the last replicated collection. Doing so could lead to full re-seeding at the next
                                  replication.
    """
    def bulk_create(self, replicate, snap_vol_list, vss_snap, **kwargs):
        """Create snapshots on the given set of volumes.

        # Parameters
        snap_vol_list : List of volumes to snapshot and corresponding snapshot creation attributes. VSS application-synchronized snapshot must specify the 'writable' parameter and
                        set it to true.
        replicate     : Allow snapshot to be replicated.
        vss_snap      : VSS app-synchronized snapshot; we don't support creation of non app-synchronized sanpshots through this interface; must be set to true.
        """

        return self._collection.bulk_create(
            self,
            replicate,
            snap_vol_list,
            vss_snap,
            **kwargs
        )


class SnapshotList(Collection):
    resource = Snapshot
    resource_type = "snapshots"

    def bulk_create(self, replicate, snap_vol_list, vss_snap, **kwargs):
        """Create snapshots on the given set of volumes.

        # Parameters
        snap_vol_list : List of volumes to snapshot and corresponding snapshot creation attributes. VSS application-synchronized snapshot must specify the 'writable' parameter and
                        set it to true.
        replicate     : Allow snapshot to be replicated.
        vss_snap      : VSS app-synchronized snapshot; we don't support creation of non app-synchronized sanpshots through this interface; must be set to true.
        """

        return self._client.perform_bulk_resource_action(
            self.resource_type,
            'bulk_create',
            replicate=replicate,
            snap_vol_list=snap_vol_list,
            vss_snap=vss_snap,
            **kwargs
        )



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient.v1 import client as nimclient
from nimbleclient import exceptions
import tests.test_protection_schedules as prot_sched
import tests.test_snapshots as snapshot
import tests.test_volume as volume
import tests.test_volume_collection as volcoll
import time

# global variables
volcoll_name1 = nimosclientbase.get_unique_string("replicationtc-volcoll1")
# snapcoll_name1 = nimosclientbase.get_unique_string("replicationtc-snapcoll1")
snapshot_name1 = nimosclientbase.get_unique_string("replicationtc-snapshot1")
vol_name1 = nimosclientbase.get_unique_string("replicationtc-vol1")
downstream_partner_id = ""
# constants
REPLICATION_ARRAY_CREDENTIALS = "ReplicationPartnerArrayCredentials"

'''ReplicationPartnerTestCase tests the replication partner
    object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for ReplicationPartner TestCase  *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for ReplicationPartner TestCase  *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    global volcoll_name1, snapshot_name1, vol_name1
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    snapshot.delete_snapshot()
    volume.delete_volume()
    volcoll.delete_volcoll()
    remove_replication_partner()
    # create new name for next test
    volcoll_name1 = nimosclientbase.get_unique_string("replicationtc-volcoll1")
    # snapcoll_name1 = nimosclientbase.get_unique_string(
    #     "replicationtc-snapcoll1")
    snapshot_name1 = nimosclientbase.get_unique_string(
        "replicationtc-snapshot1")
    vol_name1 = nimosclientbase.get_unique_string("replicationtc-vol1")
    nimosclientbase.log_footer(request.function.__name__)


# get the replication partner (downstream) client from config file
def get_downstream_client():
    # read the config which contains array credentials
    arraydetail = nimosclientbase.get_config_section_detail(
        REPLICATION_ARRAY_CREDENTIALS)
    if(arraydetail.__len__() == 3):
        rep_osclient = nimclient.NimOSClient(
            arraydetail[nimosclientbase.ARRAY_HOSTNAME],
            arraydetail[nimosclientbase.ARRAY_USERNAME],
            arraydetail[nimosclientbase.ARRAY_PASSWORD])
    else:
        raise Exception("replication Array credentials "
                        "not present in config file.")
    return rep_osclient


def is_snapshot_replicated(vol_resp):
    snapshot_list_resp = nimosclientbase.get_nimos_client().snapshots.list(
        detail=True, vol_name=vol_resp.attrs.get("name"))
    assert snapshot_list_resp is not None
    for snapshot_obj in snapshot_list_resp:
        # check for replicated flag
        if snapshot_obj.attrs.get("replication_status") == "complete":
            return True
    return False


def get_hostname(config_section_name):
    # read the config which contains array credentials
    arraydetail = nimosclientbase.get_config_section_detail(
        config_section_name)
    if(arraydetail.__len__() == 3):
        return arraydetail[nimosclientbase.ARRAY_HOSTNAME]
    else:
        raise Exception("Replication Array credentials not present in "
                        "config file.")


def get_downstream_partner_detail():
    # get the group name of the downstream array
    downstream_repl_partner = get_downstream_client().groups.get()
    assert downstream_repl_partner is not None
    return downstream_repl_partner


def get_downstream_partner_id():
    downstream_id = None
    downstream_grp_repl_partner = get_downstream_partner_detail()
    # get the upstream repl partner list and check if the list
    # has downstream array details.
    upstream_repl_partner_list = nimosclientbase.get_nimos_client(
        ).replication_partners.list()

    for upstream_repl_partner_obj in upstream_repl_partner_list:
        if upstream_repl_partner_obj.attrs.get(
                "name") == downstream_grp_repl_partner.attrs.get("name"):
            downstream_grp_id = downstream_grp_repl_partner.attrs.get("id")
            grp_name = downstream_grp_repl_partner.attrs.get("name")
            log("Found downstream group Replication: "
                f" {grp_name} with id '{downstream_grp_id}'")
            # we got a  match.
            downstream_id = upstream_repl_partner_obj.attrs.get("id")
            break
    return downstream_id


def test_replication_partner_connection():
    try:

        downstream_partner_id = get_downstream_partner_id()
        if downstream_partner_id is None:
            return

        # we got a  match.
        # test the connection to downstream array.
        nimosclientbase.get_nimos_client(
        ).replication_partners.test(id=downstream_partner_id)

        upstream_grp_resp = nimosclientbase.get_nimos_client().groups.get()
        # get the downstream repl partner list and check if the list
        # has upstream array details.
        downstream_repl_partner_list = get_downstream_client(
            ).replication_partners.list()

        for downstream_repl_partner_obj in downstream_repl_partner_list:
            if downstream_repl_partner_obj.attrs.get(
               "name") == upstream_grp_resp.attrs.get("name"):
                # we got a  match.
                # test the connection to the upstream array.
                id = upstream_grp_resp.attrs.get("id")
                grp_name = upstream_grp_resp.attrs.get("name")
                log("Found upstream group Replication partner for testing "
                    f"connection {grp_name} with id '{id}'")

                get_downstream_client().replication_partners.test(
                    id=downstream_repl_partner_obj.attrs.get("id"))
    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log("Failed as expected. mandatory arguments missing")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


def add_replication_partner():
    try:
        downstream_repl_partner = get_downstream_partner_detail()
        id = downstream_repl_partner.attrs.get("id")
        grp_name = downstream_repl_partner.attrs.get("name")
        log("downstream Replication array details  "
            f"{grp_name} with id '{id}'")

        assert downstream_repl_partner is not None
        try:
            # add the details of downstream in the upstream array
            nimosclientbase.get_nimos_client(
            ).replication_partners.create(
                name=downstream_repl_partner.attrs.get("name"),
                secret="password-91",
                hostname=get_hostname(REPLICATION_ARRAY_CREDENTIALS),
                subnet_label="mgmt-data"
            )
        except exceptions.NimOSAPIError as ex:
            if "SM_eexist" in str(ex):
                log("Replication Array already added. Ignoring this step")

        # get the group name of the upstream array
        upstream_resp = nimosclientbase.get_nimos_client().groups.get()
        assert upstream_resp is not None

        # add the details of upstream to downstream array
        upstream_grp_name = upstream_resp.attrs.get("name")
        get_downstream_client(
        ).replication_partners.create(
            name=upstream_grp_name,
            secret="password-91",
            hostname=get_hostname(nimosclientbase.NIMBLE_ARRAY_CREDENTIALS),
            subnet_label="mgmt-data"
        )
        # test the connection
        test_replication_partner_connection()

    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log("Failed as expected. mandatory arguments missing")
        elif "SM_eexist" in str(ex):
            log("Replication Partner already added from before")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


def remove_replication_partner():
    try:
        downstream_repl_partner = get_downstream_partner_detail()
        if downstream_repl_partner is None:
            return
        # get the upstream group array name.
        upstream_resp = nimosclientbase.get_nimos_client(
        ).groups.get()

        # get the upstream repl partner list and check if the list
        # has downstream array details.
        upstream_repl_partner_list = nimosclientbase.get_nimos_client(
            ).replication_partners.list()
        for upstream_repl_partner_obj in upstream_repl_partner_list:
            if upstream_repl_partner_obj.attrs.get(
               "name") == downstream_repl_partner.attrs.get("name"):
                id = downstream_repl_partner.attrs.get("id")
                grp_name = downstream_repl_partner.attrs.get("name")
                log("Removing downstream group Replication partner "
                    f"{grp_name} with id '{id}'")
                # we got a  match.
                # remove the details of downstream from the upstream array.
                nimosclientbase.get_nimos_client(
                ).replication_partners.delete(
                    id=upstream_repl_partner_obj.attrs.get("id"))

        # get the downstream repl partner list and check if the list
        # has upstream array details.
        downstream_repl_partner_list = get_downstream_client(
            ).replication_partners.list()
        for downstream_repl_partner_obj in downstream_repl_partner_list:
            if downstream_repl_partner_obj.attrs.get(
               "name") == upstream_resp.attrs.get("name"):
                id = upstream_resp.attrs.get("id")
                grp_name = upstream_resp.attrs.get("name")
                log("Removing upstream group Replication partner "
                    f"{grp_name} with id '{id}'")
                # we got a  match.
                # remove the details of upstream from the downstream array.
                get_downstream_client().replication_partners.delete(
                    id=downstream_repl_partner_obj.attrs.get("id"))
    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log("Failed as expected. mandatory arguments missing")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_replication_partner_using_invalid_params(
        setup_teardown_for_each_test):
    try:
        create_resp = nimosclientbase.get_nimos_client(
            ).replication_partners.create(
            name="tcreppartner",
            secret="1234566786",
            hostname="10.18.161.229",
            subnet_label="subnet1")
        assert create_resp is not None
    except exceptions.NimOSAPIError:
        log("Failed as expected. rep partner cannot be "
            "created from testcase as no valid data present")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_check_mandatory_params_replication_partner(
        setup_teardown_for_each_test):

    try:
        create_resp = nimosclientbase.get_nimos_client(
        ).replication_partners.create(name="tcreppartner")
        assert create_resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log("Failed as expected. mandatory arguments missing")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_add_remove_replication_partner(setup_teardown_for_each_test):
    try:
        add_replication_partner()
        remove_replication_partner()
    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log("Failed as expected. mandatory arguments missing")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_replication_partner(setup_teardown_for_each_test):
    try:
        get_resp = nimosclientbase.get_nimos_client(
                    ).replication_partners.get()
        if get_resp is None:
            log("ignoring this testcase.this testcase should be run "
                "only on replication setup")
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_replication_partners_endrow_beyond(setup_teardown_for_each_test):
    try:
        nimosclientbase.get_nimos_client().replication_partners.get(endRow=30)
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected.no rows present")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


# @pytest.mark.skipif(SKIPTEST is True,
#                     reason="skipped this test as SKIPTEST variable is true")
# def test_select_fields_for_replicationpartners(setup_teardown_for_each_test):

#     try:
#         resp = nimosclientbase.get_nimos_client().replication_partners.get(
#             fields="name,hostname,repl_hostname,pool_name")
#         if resp is not None:
#             assert "name" is not None
#             assert "hostname" is not None
#             assert "repl_hostname" is not None
#             assert "pool_name" is not None
#         else:
#             log("ignoring this testcase.this testcase should be run only "
#                 "on replication setup")
#     except exceptions.NimOSAPIError as ex:
#         if "SM_end_row_beyond_total_rows" in str(ex):
#             log("Failed as expected")
#         else:
#             log(f"Failed with exception message : {str(ex)}")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_pause_resume_for_replicationpartners(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().replication_partners.get(
            fields="id,name,hostname,repl_hostname,pool_name")
        if resp is not None:
            # pause
            pause_resp = nimosclientbase.get_nimos_client(
                ).replication_partners.pause(id=resp.attrs.get("id"))
            assert pause_resp is not None
            # resume the operation
            resume_resp = nimosclientbase.get_nimos_client(
                ).replication_partners.resume(id=resp.attrs.get("id"))
            assert resume_resp is not None
        else:
            log("Ignoring this testcase.this testcase should be "
                "run only on replication setup")

    except exceptions.NimOSAPIError as ex:
        if "SM_pool_partner_pause_unsup" in str(ex):
            log("Failed as expected.Pool partner does not "
                "support pause and resume operation")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_connectivity_for_replicationpartners(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().replication_partners.get(
            fields="id,name,hostname,repl_hostname,pool_name")
        if resp is not None:
            # tests the connection
            nimosclientbase.get_nimos_client().replication_partners.test(
                id=resp.attrs.get("id"))
        else:
            log("Ignoring this testcase."
                "This testcase should be run only on replication setup")
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_async_replication_workflow(setup_teardown_for_each_test):

    # steps involved.
    # 1.check if we have replication setup.
    # 2.create a volume, volcoll and snapcoll on upstream array
    # 3.create a periodic snapshot schedule for the volcoll
    # 4.see if the snapshot got replicated
    try:
        # step 1.
        log("Creating replication setup")
        add_replication_partner()
        # step 2
        log("Creating a volume, volcoll and snapcoll on upstream array")
        volcoll_resp = volcoll.create_volcoll(volcoll_name1)
        assert volcoll_resp is not None
        vol_resp = volume.create_volume(vol_name1, size=5)
        assert vol_resp is not None
        # associate the volume to volcoll
        vol_associate_resp = nimosclientbase.get_nimos_client(
        ).volumes.associate(
            id=vol_resp.attrs.get("id"), volcoll=volcoll_resp)
        assert vol_associate_resp is not None
        # step 3
        log("Creating Replicated periodic snapshot schedule for the volcoll")
        downstream_partner_id = get_downstream_partner_id()
        log(f"Got downstream_partner_id as '{downstream_partner_id}'")
        resp = prot_sched.create_protection_schedule(
            prot_sched.prot_schedule_name1,
            description="created by testcase for replication",
            volcoll_or_prottmpl_id=volcoll_resp.attrs.get("id"),
            volcoll_or_prottmpl_type="volume_collection",
            period=1,
            period_unit="minutes",
            at_time=0,
            until_time=86399,
            days="all",
            num_retain=1,
            downstream_partner="group-alokr8-va",
            downstream_partner_id=get_downstream_partner_id(),
            replicate_every=1,
            num_retain_replica=1
        )
        assert resp is not None
        # step 4 verify if the snapshot got replicated
        # sleep for sometime and then check if snapshot is replicated
        log("Waiting for 2 minutes for snapshot to be replicated")
        time.sleep(121)
        max_retry = 3
        retry = 1
        while is_snapshot_replicated(vol_resp) is False and retry < max_retry:
            time.sleep(30)
            retry += 1
        if retry == max_retry:
            raise Exception("Snapshot was not replicated")
        else:
            log("Snapshot Successfully replicated.")
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg" in str(ex):
            log(f"Failed as expected . snapshot name {ex}")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#

import logging
import uuid
import requests
import json
import time
import datetime

from ..exceptions import NimOSAuthenticationError, NimOSAPIError, NimOSClientJobTimeoutError
from ..__init__ import __version__

requests.packages.urllib3.disable_warnings(requests.packages.urllib3.exceptions.InsecureRequestWarning)


class SessionManager:
    """Tracks current NimOS REST sessions in order to reuse them"""

    _SESSIONS = {}


class NimOSAPIClient:
    """NimOS REST API Client session"""

    _ENDPOINTS = {
        # This endpoint list was auto-generated by the Python SDK generator; DO NOT EDIT.
        'access_control_records': 'v1/access_control_records',
        'active_directory_memberships': 'v1/active_directory_memberships',
        'alarms': 'v1/alarms',
        'application_categories': 'v1/application_categories',
        'application_servers': 'v1/application_servers',
        'arrays': 'v1/arrays',
        'audit_log': 'v1/audit_log',
        'chap_users': 'v1/chap_users',
        'controllers': 'v1/controllers',
        'disks': 'v1/disks',
        'events': 'v1/events',
        'fibre_channel_configs': 'v1/fibre_channel_configs',
        'fibre_channel_initiator_aliases': 'v1/fibre_channel_initiator_aliases',
        'fibre_channel_interfaces': 'v1/fibre_channel_interfaces',
        'fibre_channel_ports': 'v1/fibre_channel_ports',
        'fibre_channel_sessions': 'v1/fibre_channel_sessions',
        'folders': 'v1/folders',
        'groups': 'v1/groups',
        'initiator_groups': 'v1/initiator_groups',
        'initiators': 'v1/initiators',
        'jobs': 'v1/jobs',
        'key_managers': 'v1/key_managers',
        'master_key': 'v1/master_key',
        'network_configs': 'v1/network_configs',
        'network_interfaces': 'v1/network_interfaces',
        'performance_policies': 'v1/performance_policies',
        'pools': 'v1/pools',
        'protection_schedules': 'v1/protection_schedules',
        'protection_templates': 'v1/protection_templates',
        'protocol_endpoints': 'v1/protocol_endpoints',
        'replication_partners': 'v1/replication_partners',
        'shelves': 'v1/shelves',
        'snapshot_collections': 'v1/snapshot_collections',
        'snapshots': 'v1/snapshots',
        'software_versions': 'v1/software_versions',
        'space_domains': 'v1/space_domains',
        'subnets': 'v1/subnets',
        'tokens': 'v1/tokens',
        'user_groups': 'v1/user_groups',
        'user_policies': 'v1/user_policies',
        'users': 'v1/users',
        'versions': 'versions',
        'volume_collections': 'v1/volume_collections',
        'volumes': 'v1/volumes',
        'witnesses': 'v1/witnesses',
    }

    def __init__(self, hostname, username, password, job_timeout=60, port=5392):
        """Initialize a session to the NimOS REST API."""

        connection_hash = str(uuid.uuid3(uuid.NAMESPACE_OID, f'{hostname}{port}{username}{password}'))

        self.hostname = hostname
        self.port = port
        self.job_timeout = job_timeout

        self.__auth = {
            'data': {
                'username': username,
                'password': password,
                'app_name': f'NimOS Python SDK v{__version__}'
            }
        }
        logging.debug(f"NimOSAPIClient created with [hostname: {hostname}], [job_timeout: {job_timeout} seconds], [port: {port}], [SDK Version: v{__version__}]")

        self.__connection_hash = connection_hash

        if connection_hash in SessionManager._SESSIONS:
            self.session_id, self.session_token = SessionManager._SESSIONS[connection_hash]
            self.connected = True
            self._headers = {'X-Auth-Token': str(self.session_token)}

        else:
            self._headers = {}
            self.session_token = None
            self.session_id = None
            self.connected = self._connect()

    def _connect(self):
        """Perform NimOS authentication and session token retrieval."""

        try:
            response = requests.post(
                f"https://{self.hostname}:{self.port}/{self._ENDPOINTS['tokens']}",
                json=self.__auth,
                verify=False
            )

            sessiondata = response.json()

            if 'messages' in sessiondata and sessiondata['messages'][0]['code'] == 'SM_http_unauthorized':
                raise NimOSAuthenticationError("Invalid credentials")

            sessiondata = sessiondata['data']

            if 'session_token' not in sessiondata:
                raise NimOSAuthenticationError("Invalid credentials")

            self.session_token = sessiondata['session_token']
            self.session_id = sessiondata['id']
            self._headers = {'X-Auth-Token': str(self.session_token)}

            SessionManager._SESSIONS[self.__connection_hash] = (self.session_id, self.session_token)

            return True

        except requests.ConnectionError as error:
            logging.exception(error)
            raise ConnectionError(f"Error connecting to {self.hostname}")

    def _refresh_connection(self):
        """Checks status of NimOS session and reconnects if necessary."""

        try:
            response = requests.get(
                f"https://{self.hostname}:{self.port}/{self._ENDPOINTS['tokens']}/{self.session_id}",
                headers=self._headers,
                verify=False
            ).json()

            if 'messages' in response and response['messages'][0]['severity'] == 'error':
                self._connect()

        except requests.ConnectionError as error:
            logging.exception(error)
            raise ConnectionError(f"Error reconnecting to {self.hostname}")

    def close_connection(self):
        """Closes NimOS session (deletes user token)"""

        try:
            requests.delete(
                f"https://{self.hostname}:{self.port}/{self._ENDPOINTS['tokens']}/{self.session_id}",
                headers=self._headers,
                verify=False
            )

            del SessionManager._SESSIONS[self.__connection_hash]

        except requests.exceptions.RequestException as error:
            logging.exception(error)
            raise ConnectionError("Error closing connection")

    @classmethod
    def build_advanced_criteria(self, operator, criteria):
        """Constructs advanced criteria and returns JSON object."""

        return {
            'data': {
                '_constructor': 'AdvancedCriteria',
                'operator': operator,
                'criteria': criteria,
            },
            'operationType': 'fetch'
        }

    def get(self, endpoint, filter=None, limit=None, from_id=None, **params):
        """Wrapper for GET requests with filters and advanced criteria"""

        # Set the url if limit and/or from_id (pagination) is requested
        url = f'https://{self.hostname}:{self.port}/{endpoint}'

        if limit is not None:
            url = f'{url}?pageSize={limit}'

        if from_id is not None:
            url = f"{url}{'&' if limit is not None else '?'}id%3E{from_id}"

        try:
            # Init response data and query for more paginated data records if available
            response_data = []
            while 1:
                if filter is not None:
                    response = requests.post(
                        url,
                        params=params,
                        json=filter,
                        headers=self._headers,
                        verify=False
                    )
                else:
                    response = requests.get(
                        url,
                        params=params,
                        headers=self._headers,
                        verify=False
                    )

                if response.status_code >= 400:
                    if 'SM_http_unauthorized' in str(response.content):
                        self._refresh_connection()
                    else:
                        raise NimOSAPIError(response.json())

                response = response.json()

                # Check for errors if any in the response (Treat partial response as an error)
                if 'messages' in response:
                    raise NimOSAPIError(response['messages'])

                # Get response
                if 'totalRows' not in response:
                    return response

                # Append to the resultant data and continue to read more
                response_data.extend(response["data"])

                # When limit/pageSize is requested, then break and return the response.
                if limit is not None:
                    break

                # No more records to read, stop here
                if 'data' in response and len(response['data']) == 0:
                    break

                # When 'endRow' is equal to 'totalRows', stop here.
                if response['endRow'] == response['totalRows']:
                    break

                # Set next start row index to continue reading more records
                if 'startRow' in params:
                    params['startRow'] += len(response["data"])
                else:
                    params['startRow'] = len(response["data"])

                logging.debug(f"[read_records_count: {len(response_data)}], [total_rows: {response['totalRows']}], [next_start_row: {params['startRow']}]")

            logging.debug(f'Retrieved {len(response_data)} record(s)')
            return response_data

        except requests.exceptions.RequestException as error:
            logging.exception(error)
            raise ConnectionError(f"Error retrieving data from {self.hostname}")

    def delete(self, endpoint, job_timeout=None):
        """Wrapper for DELETE requests"""

        logging.debug(f"job_timeout: [{job_timeout if job_timeout is not None else self.job_timeout} seconds]")
        try:
            while 1:
                response = requests.delete(
                    f'https://{self.hostname}:{self.port}/{endpoint}',
                    headers=self._headers,
                    verify=False
                )

                if response.status_code >= 400:
                    if 'SM_http_unauthorized' in str(response.content):
                        self._refresh_connection()
                    else:
                        raise NimOSAPIError(response.json())
                elif response.status_code == 202:
                    # Handle async job response
                    job_response = response.json()
                    if 'messages' in job_response and job_response['messages'][0]['code'] == 'SM_async_job_id':
                        logging.debug("Got async job response: %s", json.dumps(job_response['messages'], indent=4))
                        return self.job_handler(job_response['messages'][0]['arguments']['job_id'], job_timeout)
                    else:
                        break
                else:
                    break

            return response.json()

        except requests.exceptions.RequestException as error:
            logging.exception(error)
            raise ConnectionError(f"Error communicating with {self.hostname}")

    def put(self, endpoint, job_timeout=None, **payload):
        """Wrapper for PUT requests."""

        logging.debug(f"job_timeout: [{job_timeout if job_timeout is not None else self.job_timeout} seconds]")

        # Translate metadata dict to REST key-value pairs.
        # Ex: {'k1': 'k1', 'k2': 'v2'} ==> [{'key': 'k1', 'value': 'v1'}, {'key': 'k2', 'value': 'v2'}]
        if 'metadata' in payload:
            payload['metadata'] = [{'key': key, 'value': payload['metadata'][key]} for key in payload['metadata']]

        try:
            while 1:
                response = requests.put(
                    f'https://{self.hostname}:{self.port}/{endpoint}',
                    headers=self._headers,
                    json={'data': payload},
                    verify=False
                )

                if response.status_code >= 400:
                    if 'SM_http_unauthorized' in str(response.content):
                        self._refresh_connection()
                    else:
                        raise NimOSAPIError(response.json())
                elif response.status_code == 202:
                    # Handle async job response
                    job_response = response.json()
                    if 'messages' in job_response and job_response['messages'][0]['code'] == 'SM_async_job_id':
                        logging.debug("Got async job response: %s", json.dumps(job_response['messages'], indent=4))
                        return self.job_handler(job_response['messages'][0]['arguments']['job_id'], job_timeout)
                    else:
                        break
                else:
                    break

            return response.json()

        except requests.exceptions.RequestException as error:
            logging.exception(error)
            raise ConnectionError(f"Error communicating with {self.hostname}")

    def post(self, endpoint, job_timeout=None, **params):
        """Wrapper for POST requests"""

        logging.debug("job_timeout: [%s seconds]", job_timeout if job_timeout is not None else self.job_timeout)

        # Translate metadata dict to REST key-value pairs.
        # Ex: {'k1': 'k1', 'k2': 'v2'} ==> [{'key': 'k1', 'value': 'v1'}, {'key': 'k2', 'value': 'v2'}]
        if 'metadata' in params:
            params['metadata'] = [{'key': key, 'value': params['metadata'][key]} for key in params['metadata']]

        try:
            while 1:
                response = requests.post(
                    f'https://{self.hostname}:{self.port}/{endpoint}',
                    headers=self._headers,
                    json={'data': params},
                    verify=False
                )

                if response.status_code >= 400:
                    if 'SM_http_unauthorized' in str(response.content):
                        self._refresh_connection()
                    else:
                        raise NimOSAPIError(response.json())
                elif response.status_code == 202:
                    # Handle async job response
                    job_response = response.json()
                    if 'messages' in job_response and job_response['messages'][0]['code'] == 'SM_async_job_id':
                        logging.debug("Got async job response: %s", json.dumps(job_response['messages'], indent=4))
                        return self.job_handler(job_response['messages'][0]['arguments']['job_id'], job_timeout)
                    else:
                        break
                else:
                    break

            response = response.json()
            return response

        except requests.exceptions.RequestException as error:
            logging.exception(error)
            raise ConnectionError(f"Error communicating with {self.hostname}")

    def get_resource(self, resource, ident, **params):
        if resource not in self._ENDPOINTS:
            raise ValueError(f"Unknown resource {resource}")

        resp = self.get(f"{self._ENDPOINTS[resource]}/{ident}", **params)
        return resp['data'] if 'data' in resp else resp

    def list_resources(self, resource, detail=False, filter=None, limit=None, from_id=None, **params):
        if resource not in self._ENDPOINTS:
            raise ValueError(f"Unknown resource {resource}")

        # Build advanced criteria filter if requested
        if filter is not None:
            filter = self.build_advanced_criteria(filter['operator'], filter['criteria'])

        resp = self.get(f"{self._ENDPOINTS[resource]}{'/detail' if detail else ''}", filter, limit, from_id, **params)
        return resp['data'] if 'data' in resp else resp

    def create_resource(self, resource, **params):
        if resource not in self._ENDPOINTS:
            raise ValueError(f"Unknown resource {resource}")

        resp = self.post(self._ENDPOINTS[resource], **params)
        return resp['data'] if 'data' in resp else resp

    def delete_resource(self, resource, ident, job_timeout=None):
        if resource not in self._ENDPOINTS:
            raise ValueError(f"Unknown resource {resource}")

        resp = self.delete(f"{self._ENDPOINTS[resource]}/{ident}", job_timeout)
        return resp['data'] if 'data' in resp else resp

    def update_resource(self, resource, ident, **params):
        if resource not in self._ENDPOINTS:
            raise ValueError(f"Unknown resource {resource}")

        resp = self.put(f"{self._ENDPOINTS[resource]}/{ident}", **params)
        return resp['data'] if 'data' in resp else resp

    def perform_resource_action(self, resource, ident, action, **params):
        if resource not in self._ENDPOINTS:
            raise ValueError(f"Unknown resource {resource}")

        resp = self.post(f"{self._ENDPOINTS[resource]}/{ident}/actions/{action}", **params)
        return resp['data'] if 'data' in resp else resp

    def perform_bulk_resource_action(self, resource, action, **params):
        if resource not in self._ENDPOINTS:
            raise ValueError(f"Unknown resource {resource}")

        resp = self.post(f"{self._ENDPOINTS[resource]}/actions/{action}", **params)
        return resp['data'] if 'data' in resp else resp

    def job_handler(self, job_id, job_timeout, retry_interval=5):

        # Use global/default job timeout if unspecified in the request
        timeout_sec = job_timeout if job_timeout is not None else self.job_timeout

        now = datetime.datetime.now()
        expiry_time = now + datetime.timedelta(0, timeout_sec)
        logging.debug(f"job_id: [{job_id}], job_timeout: [{timeout_sec} seconds], current_time: [{now}]" +
                      f"job_retry_expiry_time: [{expiry_time}], retry_interval: [{retry_interval} seconds]")
        while 1:
            try:
                # Get job status from Array
                response = self.get_resource('jobs', job_id)
                logging.debug("JOB STATUS RESPONSE: %s", json.dumps(response, indent=4))
                if (response['state'] == 'done'):
                    logging.info("Job with id '%s' completed with status '%s'", job_id, response['result'])
                    break

                if datetime.datetime.now().time() > expiry_time.time():
                    logging.error("Job status retry timeout has expired after [%s] seconds, returning with client timeout exception", timeout_sec)
                    raise NimOSClientJobTimeoutError(response)
                else:
                    #  Sleep n Wait
                    if timeout_sec < retry_interval:
                        logging.debug("Sleeping for [%s seconds] before next retry attempt ...", timeout_sec)
                        time.sleep(timeout_sec)
                    else:
                        logging.debug("Sleeping for [%s seconds] before next retry attempt ...", retry_interval)
                        time.sleep(retry_interval)

            except requests.exceptions.RequestException as error:
                logging.exception(error)
                raise ConnectionError(f"Error communicating with {self.hostname}")

        return response



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from .snapshots import SnapshotList
from .volume_collections import VolumeCollection


class Volume(Resource):
    """Volumes are the basic storage units from which the total capacity is apportioned. The terms volume and LUN are used interchangeably.The number of volumes per array depends on
    storage allocation.

    # Parameters
    id                            : Identifier for the volume.
    name                          : Name of the volume.
    full_name                     : Fully qualified name of volume.
    search_name                   : Name of volume used for object search.
    size                          : Volume size in mebibytes. Size is required for creating a volume but not for cloning an existing volume.
    description                   : Text description of volume.
    perfpolicy_name               : Name of performance policy.
    perfpolicy_id                 : Identifier of the performance policy. After creating a volume, performance policy for the volume can only be changed to another performance
                                    policy with same block size.
    reserve                       : Amount of space to reserve for this volume as a percentage of volume size.
    warn_level                    : This attribute is deprecated. Alert threshold for the volume's mapped usage, expressed as a percentage of the volume's size. When the volume's
                                    mapped usage exceeds warn_level, the array issues an alert. If this option is not specified, array default volume warn level setting is used to
                                    decide the warning level for this volume.
    limit                         : Limit on the volume's mapped usage, expressed as a percentage of the volume's size. When the volume's mapped usage exceeds limit, the volume
                                    will be offlined or made non-writable. If this option is not specified, array default volume limit setting is used to decide the limit for this
                                    volume.
    snap_reserve                  : Amount of space to reserve for snapshots of this volume as a percentage of volume size.
    snap_warn_level               : Threshold for available space as a percentage of volume size below which an alert is raised.
    snap_limit                    : This attribute is deprecated. The array does not limit a volume's snapshot space usage. The attribute is ignored on input and returns max int64
                                    on output.
    snap_limit_percent            : This attribute is deprecated. The array does not limit a volume's snapshot space usage. The attribute is ignored on input and returns -1 on
                                    output.
    num_snaps                     : Number of live, non-hidden snapshots for this volume.
    projected_num_snaps           : Deprecated. Projected number of snapshots (including scheduled and manual) for this volume.
    online                        : Online state of volume, available for host initiators to establish connections.
    owned_by_group                : Name of group that currently owns the volume.
    owned_by_group_id             : ID of group that currently owns the volume.
    multi_initiator               : For iSCSI Volume Target, this flag indicates whether the volume and its snapshots can be accessed from multiple initiators at the same time.
                                    The default is false. For iSCSI Group Target or FC access protocol, the attribute cannot be modified and always reads as false.
    iscsi_target_scope            : This indicates whether volume is exported under iSCSI Group Target or iSCSI Volume Target. This attribute is only meaningful to iSCSI system.
                                    On FC system, all volumes are exported under the FC Group Target. In create operation, the volume's target type will be set by this attribute.
                                    If not specified, it will be set as the group-setting. In clone operation, the clone's target type will inherit from the parent' setting.
    pool_name                     : Name of the pool where the volume resides. Volume data will be distributed across arrays over which specified pool is defined. If pool option
                                    is not specified, volume is assigned to the default pool.
    pool_id                       : Identifier associated with the pool in the storage pool table.
    read_only                     : Volume is read-only.
    serial_number                 : Identifier associated with the volume for the SCSI protocol.
    secondary_serial_number       : Secondary identifier associated with the volume for the SCSI protocol.
    target_name                   : The iSCSI Qualified Name (IQN) or the Fibre Channel World Wide Node Name (WWNN) of the target volume.
    block_size                    : Size in bytes of blocks in the volume.
    offline_reason                : Volume offline reason.
    clone                         : Whether this volume is a clone. Use this attribute in combination with name and base_snap_id to create a clone by setting clone = true.
    parent_vol_name               : Name of parent volume.
    parent_vol_id                 : Parent volume ID.
    base_snap_name                : Name of base snapshot.
    base_snap_id                  : Base snapshot ID. This attribute is required together with name and clone when cloning a volume with the create operation.
    replication_role              : Replication role that this volume performs.
    volcoll_name                  : Name of volume collection of which this volume is a member.
    volcoll_id                    : ID of volume collection of which this volume is a member. Use this attribute in update operation to associate or dissociate volumes with or
                                    from volume collections. When associating, set this attribute to the ID of the volume collection. When dissociating, set this attribute to
                                    empty string.
    agent_type                    : External management agent type.
    force                         : Forcibly offline, reduce size or change read-only status a volume.
    creation_time                 : Time when this volume was created.
    last_modified                 : Time when this volume was last modified.
    protection_type               : Specifies if volume is protected with schedules. If protected, indicate whether replication is setup.
    last_snap                     : Last snapshot for this volume.
    last_replicated_snap          : Last replicated snapshot for this volume.
    dest_pool_name                : Name of the destination pool where the volume is moving to.
    dest_pool_id                  : ID of the destination pool where the volume is moving to.
    move_start_time               : The Start time when this volume was moved.
    move_aborting                 : This indicates whether the move of the volume is aborting or not.
    move_bytes_migrated           : The bytes of volume which have been moved.
    move_bytes_remaining          : The bytes of volume which have not been moved.
    move_est_compl_time           : The estimated time of completion of a move.
    usage_valid                   : This indicates whether usage information of volume and snapshots are valid or not.
    space_usage_level             : Indicates space usage level based on warning level.
    total_usage_bytes             : Sum of volume mapped usage and uncompressed backup data(including pending deletes) in bytes of this volume.
    vol_usage_compressed_bytes    : Compressed data in bytes for this volume.
    vol_usage_uncompressed_bytes  : Uncompressed data in bytes for this volume.
    vol_usage_mapped_bytes        : Mapped data in bytes for this volume.
    snap_usage_compressed_bytes   : Sum of compressed backup data in bytes stored in snapshots of this volume.
    snap_usage_uncompressed_bytes : Sum of uncompressed unique backup data in bytes stored in snapshots of this volume.
    snap_usage_populated_bytes    : Sum of backup data in bytes stored in snapshots of this volume without accounting for the sharing of data between snapshots.
    cache_pinned                  : If set to true, all the contents of this volume are kept in flash cache. This provides for consistent performance guarantees for all types of
                                    workloads. The amount of flash needed to pin the volume is equal to the limit for the volume.
    pinned_cache_size             : The amount of flash pinned on the volume.
    cache_needed_for_pin          : The amount of flash needed to pin the volume.
    upstream_cache_pinned         : This indicates whether the upstream volume is cache pinned or not.
    cache_policy                  : Cache policy applied to the volume.
    thinly_provisioned            : Set volume's provisioning level to thin.  Also advertises volume as thinly provisioned to initiators supporting thin provisioning. For such
                                    volumes, soft limit notification is set to initiators when the volume space usage crosses its volume_warn_level. Default is yes. The volume's
                                    space is provisioned immediately, but for advertising status, this change takes effect only for new connections to the volume. Initiators must
                                    disconnect and reconnect for the new setting to be take effect at the initiator level consistently.
    vol_state                     : Status of the volume.
    online_snaps                  : The list of online snapshots of this volume.
    num_connections               : Number of connections of this volume.
    num_iscsi_connections         : Number of iscsi connections of this volume.
    num_fc_connections            : Number of Fibre Channel connections of this volume.
    access_control_records        : List of access control records that apply to this volume.
    inherit_acl                   : In a volume clone operation, if both the parent and the clone have no external management agent (their agent_type property is "none"), then
                                    inherit_acl controls whether the clone will inherit a copy of the parent's access control list. If either the parent or the clone have an
                                    external management agent, then the clone will not inherit the parent's access control list.
    encryption_cipher             : The encryption cipher of the volume.
    app_uuid                      : Application identifier of volume.
    folder_id                     : ID of the folder holding this volume.
    folder_name                   : Name of the folder holding this volume. It can be empty.
    metadata                      : Key-value pairs that augment an volume's attributes.
    iscsi_sessions                : List of iSCSI sessions connected to this volume.
    fc_sessions                   : List of Fibre Channel sessions connected to this volume.
    caching_enabled               : Indicate caching the volume is enabled.
    previously_deduped            : Indicate whether dedupe has ever been enabled on this volume.
    dedupe_enabled                : Indicate whether dedupe is enabled.
    vpd_t10                       : The volume's T10 Vendor ID-based identifier.
    vpd_ieee0                     : The first 64 bits of the volume's EUI-64 identifier, encoded as a hexadecimal string.
    vpd_ieee1                     : The last 64 bits of the volume's EUI-64 identifier, encoded as a hexadecimal string.
    app_category                  : Application category that the volume belongs to.
    limit_iops                    : IOPS limit for this volume. If limit_iops is not specified when a volume is created, or if limit_iops is set to -1, then the volume has no IOPS
                                    limit. If limit_iops is not specified while creating a clone, IOPS limit of parent volume will be used as limit. IOPS limit should be in range
                                    [256, 4294967294] or -1 for unlimited. If both limit_iops and limit_mbps are specified, limit_mbps must not be hit before limit_iops. In other
                                    words, IOPS and MBPS limits should honor limit_iops <= ((limit_mbps MB/s * 2^20 B/MB) / block_size B).
    limit_mbps                    : Throughput limit for this volume in MB/s. If limit_mbps is not specified when a volume is created, or if limit_mbps is set to -1, then the
                                    volume has no MBPS limit. MBPS limit should be in range [1, 4294967294] or -1 for unlimited. If both limit_iops and limit_mbps are specified,
                                    limit_mbps must not be hit before limit_iops. In other words, IOPS and MBPS limits should honor limit_iops <= ((limit_mbps MB/s * 2^20 B/MB) /
                                    block_size B).
    needs_content_repl            : Indicates whether the volume needs content based replication.
    content_repl_errors_found     : Indicates whether the last content based replication had errors.
    last_content_snap_br_cg_uid   : The branch cg uid of the content based snapshot that was last replicated.
    last_content_snap_br_gid      : The branch gid of the content based snapshot that was last replicated.
    last_content_snap_id          : The ID of the content based snapshot that was last replicated.
    cksum_last_verified           : Last checksum verification time.
    pre_filter                    : Pre-filtering criteria.
    avg_stats_last_5mins          : Average statistics in last 5 minutes.
    srep_last_sync                : Time when synchronously replicated volume was last synchronized.
    srep_resync_percent           : Percentage of resync progress for synchronously replicated volume.
    """
    def restore(self, base_snap_id, **kwargs):
        """Restore volume data from a previous snapshot.

        # Parameters
        id           : ID of the restored volume.
        base_snap_id : ID of the snapshot to which this the volume is restored.
        """

        return self._collection.restore(
            self.id,
            base_snap_id,
            **kwargs
        )
    def move(self, dest_pool_id, **kwargs):
        """Move a volume and its related volumes to another pool. To change a single volume's folder assignment (while remaining in the same pool), use a volume update operation to
        change the folder_id attribute.

        # Parameters
        id           : ID of the volume to move.
        dest_pool_id : ID of the destination pool or folder. Specify a pool ID when the volumes should not be in a folder; otherwise, specify a folder ID and the pool will be
                       derived from the folder.
        force_vvol   : Forcibly move a Virtual Volume. Moving Virtual Volume is disruptive to the vCenter, hence it should only be done by the VASA Provider (VP). This flag should
                       only be set by the VP when it calls this API.
        """

        return self._collection.move(
            self.id,
            dest_pool_id,
            **kwargs
        )
    def bulk_move(self, dest_pool_id, vol_ids, **kwargs):
        """Move volumes and their related volumes to another pool. To change a single volume's folder assignment (while remaining in the same pool), use a volume update operation to
        change the folder_id attribute.

        # Parameters
        vol_ids      : IDs of the volumes to move.
        dest_pool_id : ID of the destination pool or folder. Specify a pool ID when the volumes should not be in a folder; otherwise, specify a folder ID and the pool will be
                       derived from the folder.
        force_vvol   : Forcibly move a Virtual Volume. Moving Virtual Volume is disruptive to the vCenter, hence it should only be done by the VASA Provider (VP). This flag should
                       only be set by the VP when it calls this API.
        """

        return self._collection.bulk_move(
            self,
            dest_pool_id,
            vol_ids,
            **kwargs
        )
    def abort_move(self, **kwargs):
        """Abort the in-progress move of the specified volume to another pool.

        # Parameters
        id : ID of the volume to stop moving to a different pool.
        """

        return self._collection.abort_move(
            self.id,
            **kwargs
        )
    def bulk_set_dedupe(self, dedupe_enabled, vol_ids, **kwargs):
        """Enable or disable dedupe on a list of volumes. If the volumes are not dedupe capable, the operation will fail for the specified volume.

        # Parameters
        vol_ids        : IDs of the volumes to enable/disable dedupe.
        dedupe_enabled : Dedupe property to be applied to the list of volumes.
        """

        return self._collection.bulk_set_dedupe(
            self,
            dedupe_enabled,
            vol_ids,
            **kwargs
        )
    def bulk_set_online_and_offline(self, online, vol_ids, **kwargs):
        """Bring a list of volumes online or offline.

        # Parameters
        vol_ids : IDs of the volumes to set online or offline.
        online  : Desired state of the volumes. "true" for online, "false" for offline.
        """

        return self._collection.bulk_set_online_and_offline(
            self,
            online,
            vol_ids,
            **kwargs
        )

    def online(self):
        """Bring volume online."""

        return self._collection.online(
            self.id
        )

    def offline(self):
        """Take volume offline."""

        return self._collection.offline(
            self.id
        )

    def associate(self, volcoll):
        """Associate the volume to a volume_collection."""

        return self._collection.associate(
            self.id,
            volcoll
        )

    def dissociate(self):
        """Dissociate the volume from a volume collection."""

        return self._collection.dissociate(
            self.id
        )

    @property
    def snapshots(self):
        return SnapshotList(
            self._client
        )


class VolumeList(Collection):
    resource = Volume
    resource_type = "volumes"

    def restore(self, id, base_snap_id, **kwargs):
        """Restore volume data from a previous snapshot.

        # Parameters
        id           : ID of the restored volume.
        base_snap_id : ID of the snapshot to which this the volume is restored.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'restore',
            base_snap_id=base_snap_id,
            id=id,
            **kwargs
        )

    def move(self, id, dest_pool_id, **kwargs):
        """Move a volume and its related volumes to another pool. To change a single volume's folder assignment (while remaining in the same pool), use a volume update operation to
        change the folder_id attribute.

        # Parameters
        id           : ID of the volume to move.
        dest_pool_id : ID of the destination pool or folder. Specify a pool ID when the volumes should not be in a folder; otherwise, specify a folder ID and the pool will be
                       derived from the folder.
        force_vvol   : Forcibly move a Virtual Volume. Moving Virtual Volume is disruptive to the vCenter, hence it should only be done by the VASA Provider (VP). This flag should
                       only be set by the VP when it calls this API.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'move',
            dest_pool_id=dest_pool_id,
            id=id,
            **kwargs
        )

    def bulk_move(self, dest_pool_id, vol_ids, **kwargs):
        """Move volumes and their related volumes to another pool. To change a single volume's folder assignment (while remaining in the same pool), use a volume update operation to
        change the folder_id attribute.

        # Parameters
        vol_ids      : IDs of the volumes to move.
        dest_pool_id : ID of the destination pool or folder. Specify a pool ID when the volumes should not be in a folder; otherwise, specify a folder ID and the pool will be
                       derived from the folder.
        force_vvol   : Forcibly move a Virtual Volume. Moving Virtual Volume is disruptive to the vCenter, hence it should only be done by the VASA Provider (VP). This flag should
                       only be set by the VP when it calls this API.
        """

        return self._client.perform_bulk_resource_action(
            self.resource_type,
            'bulk_move',
            dest_pool_id=dest_pool_id,
            vol_ids=vol_ids,
            **kwargs
        )

    def abort_move(self, id, **kwargs):
        """Abort the in-progress move of the specified volume to another pool.

        # Parameters
        id : ID of the volume to stop moving to a different pool.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'abort_move',
            id=id,
            **kwargs
        )

    def bulk_set_dedupe(self, dedupe_enabled, vol_ids, **kwargs):
        """Enable or disable dedupe on a list of volumes. If the volumes are not dedupe capable, the operation will fail for the specified volume.

        # Parameters
        vol_ids        : IDs of the volumes to enable/disable dedupe.
        dedupe_enabled : Dedupe property to be applied to the list of volumes.
        """

        return self._client.perform_bulk_resource_action(
            self.resource_type,
            'bulk_set_dedupe',
            dedupe_enabled=dedupe_enabled,
            vol_ids=vol_ids,
            **kwargs
        )

    def bulk_set_online_and_offline(self, online, vol_ids, **kwargs):
        """Bring a list of volumes online or offline.

        # Parameters
        vol_ids : IDs of the volumes to set online or offline.
        online  : Desired state of the volumes. "true" for online, "false" for offline.
        """

        return self._client.perform_bulk_resource_action(
            self.resource_type,
            'bulk_set_online_and_offline',
            online=online,
            vol_ids=vol_ids,
            **kwargs
        )

    def online(self, id):
        """Bring volume online.

        # Parameters
        id : ID of the volume.
        """

        return self._client.update_resource(
            self.resource_type,
            id,
            online=True
        )

    def offline(self, id):
        """Take volume offline.

        # Parameters
        id : ID of the volume.
        """

        return self._client.update_resource(
            self.resource_type,
            id,
            online=False
        )

    def associate(self, id, volcoll):
        """Associate the volume to a volume_collection

        # Parameters
        id : ID of the volume.
        """

        return self._client.update_resource(
            self.resource_type,
            id,
            volcoll_id=volcoll.id if isinstance(volcoll, VolumeCollection) else volcoll
        )

    def dissociate(self, id):
        """Dissociate the volume from a volume collection

        # Parameters
        id : ID of the volume.
        """

        return self._client.update_resource(
            self.resource_type,
            id,
            volcoll_id=''
        )



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

'''PoolTestCase tests the Pool object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Pool TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Pool TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_pools(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().pools.list(
        detail=True, limit=2)
    assert resp is not None
    # doc shows it has 13 properties.but in replication setup it is 43.
    # just check the length
    assert resp[0].attrs.__len__() >= 43


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_create_pools(setup_teardown_for_each_test):
    array_list = [
        {
            "id": "0900000000000004d3000000000000000000000003",
            "array_id": "0900000000000004d3000000000000000000000003"
        }
    ]
    try:
        resp = nimosclientbase.get_nimos_client().pools.create(
            name="pooltest", array_list=array_list)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_array_not_found" in str(ex):
            log("Failed as expected. Array id given is invalid")
        elif "SM_srep_group_unsup" in str(ex):
            log("Failed as expected. Synchronous replication setup "
                "requires a group with exactly two single-array pools")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_pools(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().pools.get()
        assert resp is not None
        desc_name = resp.attrs.get("description")
        name = resp.attrs.get("name")
        # update name
        update_resp = nimosclientbase.get_nimos_client().pools.update(
            resp.attrs.get("id"),
            description="modified by testcase",
            name="testcasename")
        assert update_resp is not None
        # assert it got updated
        assert update_resp.attrs.get("description") == "modified by testcase"
        assert update_resp.attrs.get("name") == "testcasename"
        # rename it back
        update_resp = nimosclientbase.get_nimos_client().pools.update(
            resp.attrs.get("id"),
            description=desc_name,
            name=name)
        assert update_resp is not None
    except Exception as ex:
        # fiji and below throws SM_eperm as exception
        if "SM_eperm" in str(ex):
            log("Failed as expected")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class FibreChannelSession(Resource):
    """Fibre Channel session is created when Fibre Channel initiator connects to this group.

    # Parameters
    id             : Unique identifier of the Fibre Channel session.
    initiator_info : Information about the Fibre Channel initiator.
    target_info    : Information about the Fibre Channel target.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class FibreChannelSessionList(Collection):
    resource = FibreChannelSession
    resource_type = "fibre_channel_sessions"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class Pool(Resource):
    """Manage pools. Pools are an aggregation of arrays.

    # Parameters
    id                            : Identifier for the pool.
    name                          : Name of pool.
    full_name                     : Fully qualified name of pool.
    search_name                   : Name of pool used for object search.
    description                   : Text description of pool.
    creation_time                 : Time when this pool was created.
    last_modified                 : Time when this pool was last modified.
    capacity                      : Total storage space of the pool in bytes.
    usage                         : Used space of the pool in bytes.
    savings                       : Overall space usage savings in the pool.
    savings_data_reduction        : Space usage savings in the pool that does not include thin-provisioning savings.
    savings_compression           : Space usage savings in the pool due to compression.
    savings_dedupe                : Space usage savings in the pool due to deduplication.
    savings_clone                 : Space usage savings in the pool due to cloning of volumes.
    savings_vol_thin_provisioning : Space usage savings in the pool due to thin provisioning of volumes.
    reserve                       : Reserved space of the pool in bytes. Sum of volume reserve in the pool.
    unused_reserve                : Unused reserve space of the pool in bytes.
    free_space                    : Free space of the pool in bytes.
    cache_capacity                : Total usable cache capacity of the pool in bytes.
    pinnable_cache_capacity       : Total pinnable cache capacity of the pool in bytes.
    pinned_cache_capacity         : Total pinned cache capacity of the pool in bytes.
    dedupe_capacity_bytes         : The dedupe capacity of a hybrid pool. Does not apply to all-flash pools.
    dedupe_usage_bytes            : The dedupe usage of a hybrid pool. Does not apply to all-flash pools.
    savings_ratio                 : Overall space usage savings in the pool expressed as ratio.
    data_reduction_ratio          : Space usage savings in the pool expressed as ratio that does not include thin-provisioning savings.
    compression_ratio             : Compression savings for the pool expressed as ratio.
    dedupe_ratio                  : Dedupe savings for the pool expressed as ratio.
    clone_ratio                   : Clone savings for the pool expressed as ratio.
    vol_thin_provisioning_ratio   : Thin provisioning savings for volumes in the pool expressed as ratio.
    snapcoll_count                : Snapshot collection count.
    snap_count                    : Snapshot count.
    array_count                   : Number of arrays in the pool.
    vol_count                     : Number of volumes assigned to the pool.
    array_list                    : List of arrays in the pool with detailed information. When create/update array list, only arrays' ID is required.
    unassigned_array_list         : List of arrays being unassigned from the pool.
    vol_list                      : The list of volumes in the pool.
    pinned_vol_list               : The list of pinned volumes in the pool.
    folder_list                   : The list of fully qualified names of folders in the pool.
    force                         : Forcibly delete the specified pool even if it contains deleted volumes whose space is being reclaimed. Forcibly remove an array from array_list
                                    via an update operation even if the array is not reachable. There should no volumes currently in the pool for the forced update operation to
                                    succeed.
    usage_valid                   : Indicates whether the usage of pool is valid.
    uncompressed_vol_usage_bytes  : Uncompressed usage of volumes in the pool.
    uncompressed_snap_usage_bytes : Uncompressed usage of snapshots in the pool.
    all_flash                     : Indicate whether the pool is an all_flash pool.
    dedupe_capable                : Indicates whether the pool is capable of hosting deduped volumes.
    dedupe_all_volumes_capable    : Indicates whether the pool can enable dedupe by default.
    dedupe_all_volumes            : Indicates if dedupe is enabled by default for new volumes on this pool.
    is_default                    : Indicates if this is the default pool.
    """
    def merge(self, target_pool_id, **kwargs):
        """Merge the specified pool into the target pool. All volumes on the specified pool are moved to the target pool and the specified pool is then deleted. All the arrays in the
        pool are assigned to the target pool.

        # Parameters
        id             : ID of the specified pool.
        target_pool_id : ID of the target pool.
        force          : Forcibly merge the specified pool into target pool.
        """

        return self._collection.merge(
            self.id,
            target_pool_id,
            **kwargs
        )


class PoolList(Collection):
    resource = Pool
    resource_type = "pools"

    def merge(self, id, target_pool_id, **kwargs):
        """Merge the specified pool into the target pool. All volumes on the specified pool are moved to the target pool and the specified pool is then deleted. All the arrays in the
        pool are assigned to the target pool.

        # Parameters
        id             : ID of the specified pool.
        target_pool_id : ID of the target pool.
        force          : Forcibly merge the specified pool into target pool.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'merge',
            id=id,
            target_pool_id=target_pool_id,
            **kwargs
        )



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

"""Test base class for Nimos Client."""

from nimbleclient.v1 import client
import datetime
import time
import configparser
import os
import threading

# global variable

# Below variable "SKIPTEST" is to be used if a user wants to just run one
# particular function. They should set the value of this to 1 on command
# prompt and then change the value of SKIPTEST to false for the function they
# wish to debug. If they want to skip the entire tests in this testcase, then
# easiest way is to change the value os.getenv('SKIPTEST', '0') TO
# os.getenv('SKIPTEST', '1') "set SKIPTEST=1"
SKIPTEST = bool(int(os.getenv('SKIPTEST', "0")))
array_version = None
log_folder = os.path.abspath(os.path.dirname(__file__)) + "//" + "logs".strip()

try:
    if os.path.exists(log_folder) is False:
        # create the log folder
        os.mkdir(log_folder)
    testcase_run_log_file = os.path.abspath(
        log_folder) + "//" + "TestcaseRun.log".strip()
    print(f"log path {testcase_run_log_file}")
    try:
        # create a thread lock to make sure only one thread writes to a file
        lock = threading.RLock()
        log_file = open(testcase_run_log_file, 'w')
    except Exception:
        pass
except Exception:
    pass

# constant variables
NIMBLE_ARRAY_CREDENTIALS = "NimbleArrayCredentials"
ARRAY_HOSTNAME = "hostname"
ARRAY_USERNAME = "username"
ARRAY_PASSWORD = "password"


config_path = os.path.join(os.path.abspath(
    os.path.dirname(__file__)), "config.ini")
os_client = None


def is_array_version_above_or_equal(arr_version_to_check):
    global array_version
    if array_version is None or arr_version_to_check is None:
        return False
    arr_version = array_version.split('.')
    version_to_check = arr_version_to_check.split('.')
    if arr_version[0] > version_to_check[0]:
        return True
    elif arr_version[0] >= version_to_check[0] and arr_version[1] >= version_to_check[1]:
        return True
    return False


def get_array_version():
    global array_version
    resp = get_nimos_client().arrays.get()
    assert resp is not None
    array_version = resp.attrs.get("version")
    assert array_version is not None
    temp = array_version.split('-')
    array_version = temp[0]


def get_unique_string(baseName):
    lock.acquire()
    time.sleep(0.2)
    unique_string = baseName + datetime.datetime.now().strftime(
        "-%d-%m-%Y") + \
        str(time.time())
    lock.release()
    return unique_string


def get_config_section_detail(section_name):
    """Function reads all the values present in the given section
        and returns it as dict."""
    to_return = {}
    config = configparser.ConfigParser()
    config.read(config_path)
    for section in config.sections():
        # log(section)
        if(section_name == section):
            for option in config.options(section):
                to_return[option] = config.get(section_name, option)
    return to_return

# below function is designed for a singleton pattern.
# ideally we should not be creating a new instance for each request.with async_job_changes to sdk
# a new instance will be created each time if job_timeout is not 60. hence, any request which wants a different job
# timeout should simply pass the desired value and a new instance will be returned to the caller


def get_nimos_client(job_timeout=60):
    global os_client
    if os_client is None or job_timeout != 60:
        # Read the config which contains array credentials
        array_detail = get_config_section_detail(NIMBLE_ARRAY_CREDENTIALS)
        if(array_detail.__len__() == 3):
            os_client = client.NimOSClient(
                array_detail[ARRAY_HOSTNAME],
                array_detail[ARRAY_USERNAME],
                array_detail[ARRAY_PASSWORD],
                job_timeout
            )
        else:
            raise Exception("Array Credentials not present in config file.")
    return os_client


def log_header(name):
    """Function to print header unit tests."""
    # log("\n##Started testing '%s' at " %
    #       name, datetime.datetime.now().strftime("%d-%m-%Y %H:%M:%S"))
    lock.acquire()
    temp = "##Started testing '{name}' at {time}".format(
        name=name, time=datetime.datetime.now().strftime("%d-%m-%Y %H:%M:%S"))
    log_to_file(temp)
    lock.release()


def log_footer(name):
    """Function to print header unit tests."""
    lock.acquire()
    temp = "##Completed testing '{name}' \n".format(name=name)
    # log("##Completed testing '%s' \n" % name)
    log_to_file(temp)
    log_file.flush()
    lock.release()


def log_to_file(tolog):
    lock.acquire()
    print("%s" % tolog)
    # out, err = capfd.readouterr()
    time = " " + datetime.datetime.now().strftime("%d-%m-%Y %H:%M:%S")
    tolog = "\n" + time + " " + str(tolog)
    log_file.write(tolog)
    lock.release()

get_array_version()


# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions
import tests.test_volume as volume
import tests.test_volume_collection as volcoll

# global variables
volcoll_name1 = nimosclientbase.get_unique_string("snapcolltc-volcoll1")
snapcoll_name1 = nimosclientbase.get_unique_string("snapcolltc-snapcoll1")
vol_name1 = nimosclientbase.get_unique_string("snapcolltc-vol1")
snapcoll_to_delete = []

'''SnapCollTestCase class tests the snapshot collection object functionality'''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Snapcoll TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Snapcoll TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    global vol_name1, snapcoll_name1, volcoll_name1
    volcoll_name1 = nimosclientbase.get_unique_string("snapcolltc-volcoll1")
    snapcoll_name1 = nimosclientbase.get_unique_string("snapcolltc-snapcoll1")
    vol_name1 = nimosclientbase.get_unique_string("snapcolltc-vol1")
    # 1st create a volcoll and 2nd associate a volume to volcoll
    volcoll_resp = volcoll.create_volcoll(volcoll_name1)
    assert volcoll_resp is not None
    # save the volcoll_id here
    volcoll_resp.attrs.get("id")
    # create and associate a volume
    volresp = volume.create_volume(vol_name1)
    try:
        nimosclientbase.get_nimos_client().volumes.update(
            id=volresp.attrs.get("id"),
            volcoll_id=volcoll_resp.attrs.get("id"))
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_snapcoll()
    volume.delete_volume()
    volcoll.delete_volcoll()
    nimosclientbase.log_footer(request.function.__name__)


def delete_snapcoll():
    for snapcoll_id in snapcoll_to_delete:
        try:
            resp = nimosclientbase.get_nimos_client(
            ).snapshot_collections.delete(id=snapcoll_id)
            assert resp is not None
            log(f"Deleted snapcoll with id '{snapcoll_id}'")
        except exceptions.NimOSAPIError as ex:
            log(f"Failed with exception message : {str(ex)}")
            raise ex
    snapcoll_to_delete.clear()


def create_snapcoll(snapcoll_name, **kwargs):
    resp = nimosclientbase.get_nimos_client().snapshot_collections.create(
        name=snapcoll_name, **kwargs)
    snapcoll_id = resp.attrs.get("id")
    snapcoll_to_delete.append(snapcoll_id)
    assert resp is not None
    log(f"Created snapcoll with name '{snapcoll_name}' and Id '{snapcoll_id}'")
    return resp


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_create_snapcoll(setup_teardown_for_each_test):
    snapcoll_resp = create_snapcoll(
        snapcoll_name1,
        volcoll_id=volcoll.volcoll_to_delete[0],
        description="created by testcase",
        replicate=False,
        start_online=False)
    assert snapcoll_resp is not None
    assert snapcoll_name1 == snapcoll_resp.attrs.get("name")
    assert "created by testcase" == snapcoll_resp.attrs.get("description")
    assert snapcoll_resp.attrs.get("replicate") is False
    # make the snapcoll offline
    resp = nimosclientbase.get_nimos_client().snapshot_collections.update(
        snapcoll_resp.attrs.get("id"), description="modified by testcase")
    assert resp is not None
    assert "modified by testcase" == resp.attrs.get("description")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_snapcoll_with_onlinesnapshot(setup_teardown_for_each_test):

    snapcoll_resp = create_snapcoll(
        snapcoll_name1,
        volcoll_id=volcoll.volcoll_to_delete[0],
        description="created by testcase",
        replicate=False,
        start_online=True)
    assert snapcoll_resp is not None
    assert snapcoll_name1 == snapcoll_resp.attrs.get("name")
    assert "created by testcase" == snapcoll_resp.attrs.get("description")
    assert snapcoll_resp.attrs.get("replicate") is False
    # get the snapcoll
    snapcoll_resp = nimosclientbase.get_nimos_client(
    ).snapshot_collections.get(id=snapcoll_resp.attrs.get("id"))
    assert snapcoll_resp
    # check whether snapcoll has snapshots created
    # assert snapcoll_resp.attrs.get("start_online"),true) may be array bug
    # as doc says this field should be there
    assert len(snapcoll_resp.attrs.get("snapshots_list")) == 1
    # offline the snapshot
    snapshotlist = snapcoll_resp.attrs.get("snapshots_list")
    for snapid in snapshotlist:
        nimosclientbase.get_nimos_client().snapshots.update(
            id=snapid["id"], online=False)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_delete_snapcoll_with_onlinesnapshot(setup_teardown_for_each_test):
    # should throw ex "SM_vol_has_online_snap"
    snapcoll_resp = create_snapcoll(
        snapcoll_name1,
        volcoll_id=volcoll.volcoll_to_delete[0],
        description="created by testcase",
        replicate=False,
        start_online=True)  # online snapshot
    assert snapcoll_resp is not None
    assert snapcoll_name1 == snapcoll_resp.attrs.get("name")
    assert "created by testcase" == snapcoll_resp.attrs.get("description")
    assert snapcoll_resp.attrs.get("replicate") is False
    # get the snapcoll
    snapcoll_resp = nimosclientbase.get_nimos_client(
    ).snapshot_collections.get(id=snapcoll_resp.attrs.get("id"))
    assert snapcoll_resp
    # try deleting it should throw exception
    try:
        delete_snapcoll()
        volume.delete_volume()
    except exceptions.NimOSAPIError as ex:
        if "SM_vol_has_online_snap" in str(ex):
            log("Failed as expected with exception : 'SM_vol_has_online_snap'")
            # offilne the snapshot
            snapshotlist = snapcoll_resp.attrs.get("snapshots_list")
            for snapid in snapshotlist:
                snapresp = nimosclientbase.get_nimos_client(
                ).snapshots.update(id=snapid["id"], online=False)
                assert snapresp is not None
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_sync_replication_partner_for_snapcoll(setup_teardown_for_each_test):
    # this tescase only works for sync repl
    try:
        snapcoll_resp = create_snapcoll(
            snapcoll_name1,
            volcoll_id=volcoll.volcoll_to_delete[0],
            description="Created by testcase",
            replicate=True,
            start_online=False)
        assert snapcoll_resp is not None
        assert snapcoll_name1 == snapcoll_resp.attrs.get("name")
        assert "created by testcase" == snapcoll_resp.attrs.get("description")
        assert snapcoll_resp.attrs.get("start_online") is True
        assert snapcoll_resp.attrs.get("replicate") is True
    except exceptions.NimOSAPIError as ex:
        if "SM_repl_no_partner_avail" in str(ex):
            log("Failed as expected. no replication partner available")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions
import tests.test_volume as volume
import threading
import time
from nimbleclient.v1 import query_fields


# global variables
snapshot_name1 = nimosclientbase.get_unique_string("snapshottc-snapshot1")
snapshot_name2 = nimosclientbase.get_unique_string("snapshottc-snapshot2")
vol_name1 = nimosclientbase.get_unique_string("snapshottc-vol1")
snapshot_to_delete = []
snapshot_lock = threading.RLock()

'''SnapshotsTestCase class tests the snapshots object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Snapshots TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Snapshot TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    global vol_name1
    nimosclientbase.log_header(request.function.__name__)
    vol_name1 = nimosclientbase.get_unique_string("snapshottc-vol1")
    vol_resp = volume.create_volume(vol_name1)
    assert vol_resp is not None
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_snapshot()
    volume.delete_volume()
    nimosclientbase.log_footer(request.function.__name__)


def delete_snapshot():
    snapshot_lock.acquire()
    for snapshot_id in snapshot_to_delete:
        try:
            resp = nimosclientbase.get_nimos_client().snapshots.delete(
                id=snapshot_id)
            assert resp is not None
            log(f"Deleted snapshot with id '{snapshot_id}'")
        except exceptions.NimOSAPIError as ex:
            snapshot_lock.release()
            log(f"Failed with exception message : {str(ex)}")
            raise ex
    snapshot_to_delete.clear()
    snapshot_lock.release()


def create_snapshot(snapshot_name, **kwargs):
    try:
        snapshot_lock.acquire()
        resp = nimosclientbase.get_nimos_client().snapshots.create(
            name=snapshot_name, **kwargs)
        snapshot_id = resp.attrs.get("id")
        snapshot_to_delete.append(snapshot_id)
        assert resp is not None
        log(f"Created snapshot with name '{snapshot_name}' and Id "
            f"'{snapshot_id}'")
        snapshot_lock.release()
        return resp
    except Exception as ex:
        snapshot_lock.release()
        log(f"Failed with exception message : {str(ex)}")
        raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_create_snapshot(setup_teardown_for_each_test):
    snapshot_resp = create_snapshot(snapshot_name1,
                                    vol_id=volume.vol_to_delete[0],
                                    description="created by testcase",
                                    online=False,
                                    writable=False)
    assert snapshot_resp is not None
    assert snapshot_name1 == snapshot_resp.attrs.get("name")
    assert "created by testcase" == snapshot_resp.attrs.get("description")
    assert snapshot_resp.attrs.get("online") is False
    assert snapshot_resp.attrs.get("writable") is False
    # test update
    resp = nimosclientbase.get_nimos_client().snapshots.update(
        snapshot_resp.attrs.get("id"), description="modified by testcase")
    assert resp is not None
    assert "modified by testcase" == resp.attrs.get("description")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_read_snapshot(setup_teardown_for_each_test):
    snap_resp = create_snapshot(snapshot_name1,
                                vol_id=volume.vol_to_delete[0],
                                description="created by testcase",
                                online=False,
                                writable=False)
    assert snap_resp is not None
    # get the snapshot based on given volume name
    snapshot_resp = nimosclientbase.get_nimos_client().snapshots.get(id=None, vol_name=vol_name1)
    assert snapshot_name1 == snapshot_resp.attrs.get("name")
    assert "created by testcase" == snapshot_resp.attrs.get("description")
    assert snapshot_resp.attrs.get("online") is False
    assert snapshot_resp.attrs.get("writable") is False
    # test update
    resp = nimosclientbase.get_nimos_client().snapshots.update(
        snapshot_resp.attrs.get("id"), description="modified by testcase")
    assert resp is not None
    assert "modified by testcase" == resp.attrs.get("description")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_clone_from_snapshot(setup_teardown_for_each_test):
    # take multiple snapshot of a volume and then try creating a
    # clone from one of them
    snap_resp = create_snapshot(snapshot_name1,
                                vol_id=volume.vol_to_delete[0],
                                description="created by testcase",
                                online=False,
                                writable=False)
    assert snap_resp is not None
    time.sleep(2)
    snap_resp = create_snapshot(snapshot_name2,
                                vol_id=volume.vol_to_delete[0],
                                description="created by testcase",
                                online=False,
                                writable=False)
    # get the snapshot based on given volume name
    snap_list_resp = nimosclientbase.get_nimos_client().snapshots.list(vol_name=vol_name1)
    snap_obj = snap_list_resp[-1]  # get the latest snapshot
    cloned_resp = nimosclientbase.get_nimos_client().volumes.create(name="snapshottc-cloned-vol",
                                                                    base_snap_id=snap_obj.attrs.get("id"),
                                                                    clone=True)
    assert cloned_resp is not None
    assert "snapshottc-cloned-vol" == cloned_resp.attrs.get("name")
    assert cloned_resp.attrs.get("clone") is True
    # delete clone vol
    vol_id = cloned_resp.attrs.get("id")
    nimosclientbase.get_nimos_client().volumes.update(
                id=vol_id, volcoll_id="")  # disassociate
    nimosclientbase.get_nimos_client().volumes.offline(vol_id)
    nimosclientbase.get_nimos_client().volumes.delete(vol_id)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_snapshot_with_invalid_name(setup_teardown_for_each_test):
    try:
        snap_name = "-;" + snapshot_name1
        create_snapshot(snap_name,
                        vol_id=volume.vol_to_delete[0],
                        description="created by testcase",
                        online=False,
                        writable=False)
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg" in str(ex):
            log(f"Failed as expected. Invalid snapshot name {snap_name}")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_delete_onlinesnapshot(setup_teardown_for_each_test):
    # should fail with exception
    snapshot_resp = create_snapshot(snapshot_name1,
                                    vol_id=volume.vol_to_delete[0],
                                    description="created by testcase",
                                    online=True,
                                    writable=False)
    assert snapshot_resp is not None
    assert snapshot_name1 == snapshot_resp.attrs.get("name")
    assert "created by testcase" == snapshot_resp.attrs.get("description")
    assert snapshot_resp.attrs.get("online") is True
    assert snapshot_resp.attrs.get("writable") is False
    delete_snapshot()


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_bulk_create_snapshot(setup_teardown_for_each_test):
    snap_vol_list = [
        {"vol_id": "0600000000000004d3000000000000000000000007",
         "snap_name": "",
         "snap_description": "",
         "cookie": "",
         "online": False,
         "writable": True},
        {"vol_id": "0600000000000004d3000000000000000000000008",
         "snap_name": "",
         "snap_description": "",
         "cookie": "",
         "online": False,
         "writable": True}
    ]

    try:
        snapshot_resp = nimosclientbase.get_nimos_client(
        ).snapshots.bulk_create(replicate=False,
                                snap_vol_list=snap_vol_list,
                                vss_snap=False)
        assert snapshot_resp is not None
    except exceptions.NimOSAPIError as ex:
        # fiji and below throws SM_http_bad_request as exception
        if "SM_http_not_found" in str(ex) or "SM_http_bad_request" in str(ex):
            # this testcase will fail as wrong vol_id is passed.intention
            # is to make sure teh correct call goes through sdk
            log("Failed as expected. wrong vol id passed")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class ApplicationServer(Resource):
    """An application server is an external agent that collaborates with an array to manage storage resources; for example, Volume Shadow Copy Service (VSS) or VMware.

    # Parameters
    id            : Identifier for the application server.
    name          : Name for the application server.
    hostname      : Application server hostname.
    port          : Application server port number.
    username      : Application server username.
    description   : Text description of application server.
    password      : Application server password.
    server_type   : Application server type ({invalid|vss|vmware|cisco|stack_vision|container_node}).
    metadata      : Key-value pairs that augment an application server's attributes.
    creation_time : Time when this application server was created.
    last_modified : Time when this application server was last modified.
    """


class ApplicationServerList(Collection):
    resource = ApplicationServer
    resource_type = "application_servers"



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class SoftwareVersion(Resource):
    """Show the software version.

    # Parameters
    version                : Software version, used as identifier in URL.
    signature              : Keyed hash of download package.
    name                   : Name of version.
    status                 : Status of version.
    total_bytes            : Size of version.
    downloaded_bytes       : Number of bytes downloaded for the version.
    blacklist_reason       : Reason for blacklisting the version. Empty if version is not blacklisted.
    release_date           : Date when software version was released.
    is_manually_downloaded : Whether or not the version was downloaded manually.
    release_status         : Release status of software version.
    no_partial_response    : Indicate that it is not ok to provide partially available response.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class SoftwareVersionList(Collection):
    resource = SoftwareVersion
    resource_type = "software_versions"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class Array(Resource):
    """Retrieve information of specified arrays. The array is the management and configuration for the underlying physical hardware array box.

    # Parameters
    id                            : Identifier for array.
    name                          : The user provided name of the array. It is also the array's hostname.
    force                         : Forcibly delete the specified array.
    full_name                     : The array's fully qualified name.
    search_name                   : The array name used for object search.
    status                        : Reachability status of the array in the group.
    role                          : Role of the array in the group.
    group_state                   : State of the array in the group.
    pool_name                     : Name of pool to which this is a member.
    pool_id                       : ID of pool to which this is a member.
    model                         : Array model.
    serial                        : Serial number of the array.
    version                       : Software version of the array.
    is_sfa                        : True if this array supports SFA; false otherwise.
    creation_time                 : Time when this array object was created.
    last_modified                 : Time when this array object was last modified.
    usage_valid                   : Indicates whether the usage of array is valid.
    usable_capacity_bytes         : The usable capacity of the array in bytes.
    usable_cache_capacity_bytes   : The usable cache capacity of the array in bytes.
    raw_capacity_bytes            : The raw capacity of the array in bytes.
    vol_usage_bytes               : The compressed usage of volumes in array.
    vol_usage_uncompressed_bytes  : The uncompressed usage of volumes in array. This is the pre-reduced usage.
    vol_compression               : The compression rate of volumes in array expressed as ratio.
    vol_saved_bytes               : The saved space of volumes in array.
    snap_usage_bytes              : The compressed usage of snapshots in array.
    snap_usage_uncompressed_bytes : The uncompressed usage of snapshots in array. This is the pre-reduced usage.
    snap_compression              : The compression rate of snapshots in array expressed as ratio.
    snap_space_reduction          : The space reduction rate of snapshots in array expressed as ratio.
    snap_saved_bytes              : The saved space of snapshots in array.
    pending_delete_bytes          : The pending delete bytes in array.
    available_bytes               : The available space of array.
    usage                         : Used space of the array in bytes.
    all_flash                     : Whether it is an all-flash array.
    dedupe_capacity_bytes         : The dedupe capacity of a hybrid array. Does not apply to all-flash arrays.
    dedupe_usage_bytes            : The dedupe usage of a hybrid array. Does not apply to all-flash arrays.
    is_fully_dedupe_capable       : Is array fully capable to dedupe its usable capacity.
    extended_model                : Extended model of the array.
    is_supported_hw_config        : Whether it is a supported hardware config.
    gig_nic_port_count            : Count of 1G NIC Ports installed on the array.
    ten_gig_sfp_nic_port_count    : Count of SFP NIC Ports installed on the array capable of 10G, 25G or 100G speeds.
    ten_gig_t_nic_port_count      : Count of 10G BaseT NIC Ports installed on the array.
    fc_port_count                 : Count of Fibre Channel Ports installed on the array.
    public_key                    : Public key of the array.
    upgrade                       : The array upgrade data.
    create_pool                   : Whether to create associated pool during array create.
    pool_description              : Text description of the pool to be created during array creation.
    allow_lower_limits            : A True setting will allow you to add an array with lower limits to a pool with higher limits.
    ctrlr_a_support_ip            : Controller A Support IP Address.
    ctrlr_b_support_ip            : Controller B Support IP Address.
    nic_list                      : List NICs information. Used when creating an array.
    model_sub_type                : Array model sub type.
    zconf_ipaddrs                 : List of link-local zero-configuration addresses of the array.
    secondary_mgmt_ip             : Secondary management IP address for the Group.
    """
    def failover(self, **kwargs):
        """Perform a failover on the specified array.

        # Parameters
        id    : ID of the array to perform failover on.
        force : Initiate failover without performing any precheck.
        job_timeout: Job timeout in seconds.
        """

        return self._collection.failover(
            self.id,
            **kwargs
        )
    def halt(self, **kwargs):
        """Halt the specified array. Restarting the array will require physically powering it back on.

        # Parameters
        id : ID of the array to halt.
        job_timeout: Job timeout in seconds.
        """

        return self._collection.halt(
            self.id,
            **kwargs
        )
    def reboot(self, **kwargs):
        """Reboot the specified array.

        # Parameters
        id : ID of the array to reboot.
        job_timeout: Job timeout in seconds.
        """

        return self._collection.reboot(
            self.id,
            **kwargs
        )


class ArrayList(Collection):
    resource = Array
    resource_type = "arrays"

    def failover(self, id, **kwargs):
        """Perform a failover on the specified array.

        # Parameters
        id    : ID of the array to perform failover on.
        force : Initiate failover without performing any precheck.
        job_timeout: Job timeout in seconds.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'failover',
            id=id,
            **kwargs
        )

    def halt(self, id, **kwargs):
        """Halt the specified array. Restarting the array will require physically powering it back on.

        # Parameters
        id : ID of the array to halt.
        job_timeout: Job timeout in seconds.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'halt',
            id=id,
            **kwargs
        )

    def reboot(self, id, **kwargs):
        """Reboot the specified array.

        # Parameters
        id : ID of the array to reboot.
        job_timeout: Job timeout in seconds.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'reboot',
            id=id,
            **kwargs
        )



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Initiator(Resource):
    """Manage initiators in initiator groups. An initiator group has a set of initiators that can be configured as part of your ACL to access a specific volume through group
    membership.

    # Parameters
    id                      : Identifier for initiator.
    access_protocol         : Access protocol used by the initiator. Valid values are 'iscsi' and 'fc'.
    initiator_group_id      : Identifier of the initiator group that this initiator is assigned to.
    initiator_group_name    : Name of the initiator group that this initiator is assigned to.
    label                   : Unique Identifier of the iSCSI initiator. Label is required when creating iSCSI initiator.
    iqn                     : IQN name of the iSCSI initiator. Each initiator IQN name must have an associated IP address specified using the 'ip_address' attribute. You can
                              choose not to enter the IP address for an initiator if you prefer not to authenticate using both name and IP address, in this case the IP address
                              will be returned as '*'.
    ip_address              : IP address of the iSCSI initiator. Each initiator IP address must have an associated name specified using 'name' attribute. You can choose not to
                              enter the name for an initiator if you prefer not to authenticate using both name and IP address, in this case the IQN name will be returned as '*'.
    alias                   : Alias of the Fibre Channel initiator. Maximum alias length is 32 characters. Each initiator alias must have an associated WWPN specified using the
                              'wwpn' attribute. You can choose not to enter the WWPN for an initiator when using previously saved initiator alias.
    chapuser_id             : Identifier for the CHAP user.
    wwpn                    : WWPN (World Wide Port Name) of the Fibre Channel initiator. WWPN is required when creating a Fibre Channel initiator. Each initiator WWPN can have an
                              associated alias specified using the 'alias' attribute. You can choose not to enter the alias for an initiator if you prefer not to assign an
                              initiator alias.
    vp_override             : Flag to allow modifying VP created initiator groups. When set to true, user can add this initiator to a VP created initiator group.
    creation_time           : Time when this initiator group was created.
    last_modified           : Time when this initiator group was last modified.
    override_existing_alias : Forcibly add Fibre Channel initiator to initiator group by updating or removing conflicting Fibre Channel initiator aliases.
    """

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class InitiatorList(Collection):
    resource = Initiator
    resource_type = "initiators"

    def create(self, **kwargs):
        resp = self._client.create_resource(self.resource_type, **kwargs)
        return self.resource(resp['id'], resp, client=self._client, collection=self)

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#

from .client import NimOSClient
from .query_fields import (
    # This import list was auto-generated by the Python SDK generator; DO NOT EDIT.
    AccessControlRecordFields,
    ActiveDirectoryMembershipFields,
    AlarmFields,
    ApplicationCategoryFields,
    ApplicationServerFields,
    ArrayFields,
    AuditLogFields,
    ChapUserFields,
    ControllerFields,
    DiskFields,
    EventFields,
    FibreChannelConfigFields,
    FibreChannelInitiatorAliasFields,
    FibreChannelInterfaceFields,
    FibreChannelPortFields,
    FibreChannelSessionFields,
    FolderFields,
    GroupFields,
    InitiatorFields,
    InitiatorGroupFields,
    JobFields,
    KeyManagerFields,
    MasterKeyFields,
    NetworkConfigFields,
    NetworkInterfaceFields,
    PerformancePolicyFields,
    PoolFields,
    ProtectionScheduleFields,
    ProtectionTemplateFields,
    ProtocolEndpointFields,
    ReplicationPartnerFields,
    ShelfFields,
    SnapshotCollectionFields,
    SnapshotFields,
    SoftwareVersionFields,
    SpaceDomainFields,
    SubnetFields,
    TokenFields,
    UserFields,
    UserGroupFields,
    UserPolicyFields,
    VersionFields,
    VolumeCollectionFields,
    VolumeFields,
    WitnessFields,
)

__all__ = [
    "AccessControlRecordFields",
    "ActiveDirectoryMembershipFields",
    "AlarmFields",
    "ApplicationCategoryFields",
    "ApplicationServerFields",
    "ArrayFields",
    "AuditLogFields",
    "ChapUserFields",
    "ControllerFields",
    "DiskFields",
    "EventFields",
    "FibreChannelConfigFields",
    "FibreChannelInitiatorAliasFields",
    "FibreChannelInterfaceFields",
    "FibreChannelPortFields",
    "FibreChannelSessionFields",
    "FolderFields",
    "GroupFields",
    "InitiatorFields",
    "InitiatorGroupFields",
    "JobFields",
    "KeyManagerFields",
    "MasterKeyFields",
    "NetworkConfigFields",
    "NetworkInterfaceFields",
    "NimOSClient",
    "PerformancePolicyFields",
    "PoolFields",
    "ProtectionScheduleFields",
    "ProtectionTemplateFields",
    "ProtocolEndpointFields",
    "ReplicationPartnerFields",
    "ShelfFields",
    "SnapshotCollectionFields",
    "SnapshotFields",
    "SoftwareVersionFields",
    "SpaceDomainFields",
    "SubnetFields",
    "TokenFields",
    "UserFields",
    "UserGroupFields",
    "UserPolicyFields",
    "VersionFields",
    "VolumeCollectionFields",
    "VolumeFields",
    "WitnessFields",
    ]



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class SnapshotCollection(Resource):
    """Snapshot collections are collections of scheduled snapshots that are taken from volumes sharing a volume collection. Snapshot collections are replicated in the order that the
    collections were taken.

    # Parameters
    id                         : Identifier for snapshot collection.
    name                       : Name of snapshot collection.
    description                : Text description of snapshot collection.
    volcoll_name               : Volume collection name.
    volcoll_id                 : Parent volume collection ID.
    origin_name                : Origination group name/ID.
    is_replica                 : Indicates if snapshot collection was created as a replica.
    srep_owner_name            : Name of the partner where the snapshots in this snapshot collection reside.
    srep_owner_id              : ID of the partner where snapshots for this snapshot collection reside which were created by synchronous replication. Field will be null if no peer
                                 snapshot_collection was created by synchronous replication.
    peer_snapcoll_id           : ID of the peer snapshot collection created by synchronous replication. Field will be null if no peer snapshot_collection was created by
                                 synchronous replication.
    num_snaps                  : Current number of live, non-hidden snaps in this collection.
    is_complete                : Is complete.
    is_manual                  : Is manual.
    is_external_trigger        : Is externally triggered.
    is_unmanaged               : Indicates whether a snapshot collection is unmanaged. This is based on the state of individual snapshots.
    is_manually_managed        : Indicates whether a snapshot collection is managed.
    repl_status                : Replication status.
    repl_start_time            : Replication start time.
    repl_complete_time         : Replication complete time.
    repl_bytes_transferred     : Number of bytes transferred after replication completes.
    creation_time              : Time when this snapshot collection was created.
    last_modified              : Time when this snapshot collection was last modified.
    online_status              : Online status of snapcoll. This is based on the online status of the individual snapshots.
    vol_snap_attr_list         : List of snapshot attributes for snapshots being created as part of snapshot collection creation.
    snapshots_list             : List of snapshots in the snapshot collection.
    replicate                  : True if this snapshot collection has been marked for replication. This attribute cannot be updated for synchronous replication.
    replicate_to               : Specifies the partner name that the snapshots in this snapshot collection are replicated to.
    start_online               : Start with snapshot set online.
    allow_writes               : Allow applications to write to created snapshot(s). Mandatory and must be set to 'true' for VSS application synchronized snapshots.
    disable_appsync            : Do not perform application synchronization for this snapshot, create a crash-consistent snapshot instead.
    snap_verify                : Run verification tool on this snapshot. This option can only be used with a volume collection that has application synchronization.
    skip_db_consistency_check  : Skip consistency check for database files on this snapshot. This option only applies to volume collections with application synchronization set to
                                 VSS, application ID set to MS Exchange 2010 or later with Database Availability Group (DAG), snap_verify option set to true, and disable_appsync
                                 option set to false.
    sched_id                   : ID of protection schedule of snapshot collection.
    sched_name                 : Name of protection schedule of snapshot collection.
    invoke_on_upstream_partner : Invoke snapshot request on upstream partner. This operation is not supported for synchronous replication volume vollections.
    agent_type                 : External management agent type for snapshots being created as part of snapshot collection.
    expiry_after               : Number of seconds after which this snapcoll is considered expired by the snapshot TTL. A value of 0 indicates that the snapshot never expires, 1
                                 indicates that the snapshot uses a group-level configured TTL value and any other value indicates the number of seconds.
    metadata                   : Key-value pairs that augment a snapshot collection's attributes.
    force                      : Forcibly delete the specified snapshot collection even if it is the last replicated snapshot. Doing so could lead to full re-seeding at the next
                                 replication.
    """


class SnapshotCollectionList(Collection):
    resource = SnapshotCollection
    resource_type = "snapshot_collections"



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#


def and_(*args):
    """Build AND filters."""

    return {
                'operator': 'and',
                'criteria': list(args)
           }


def or_(*args):
    """Build OR filters."""

    return {
                'operator': 'or',
                'criteria': list(args)
           }


class _Field:
    """Builds a simple filter criteria."""

    def __init__(self, name, value=None):
        self.name = name

    def _get_value(self, value):
        if value is True:
            value = 'true'
        elif value is False:
            value = 'false'
        elif value is None:
            value = 'null'
        else:
            value = str(value)
        return value

    def __eq__(self, value):
        return {"fieldName": self.name, "operator": "equals", "value": self._get_value(value)}

    def __ne__(self, value):
        return {"fieldName": self.name, "operator": "notEqual", "value": self._get_value(value)}

    def __ge__(self, value):
        return {"fieldName": self.name, "operator": "greaterOrEqual", "value": value}

    def __gt__(self, value):
        return {"fieldName": self.name, "operator": "greaterThan", "value": value}

    def __lt__(self, value):
        return {"fieldName": self.name, "operator": "lessThan", "value": value}

    def __le__(self, value):
        return {"fieldName": self.name, "operator": "lessOrEqual", "value": value}

    def __in__(self, value):
        return {"fieldName": self.name, "operator": "iContains", "value": str(value)}

    def __contains__(self, value):
        return {"fieldName": self.name, "operator": "iContains", "value": str(value)}

    def eq(self, value, case_sensitive=True):
        return {"fieldName": self.name, "operator": "equals" if case_sensitive else "iEquals", "value": value}

    def ne(self, value, case_sensitive=True):
        return {"fieldName": self.name, "operator": "notEqual" if case_sensitive else "iNotEqual", "value": value}

    def contains(self, value, case_sensitive=True):
        return {"fieldName": self.name, "operator": "contains" if case_sensitive else "iContains", "value": str(value)}

    def in_set(self, values):
        return {"fieldName": self.name, "operator": "inSet", "values": list(values)}

    def not_in_set(self, values):
        return {"fieldName": self.name, "operator": "notInSet", "values": list(values)}

    def between(self, value1, value2):
        return {"fieldName": self.name, "operator": "between", "value1": value1, "value2": value2}

    def between_inclusive(self, value1, value2):
        return {"fieldName": self.name, "operator": "betweenInclusive", "value1": value1, "value2": value2}

    def regex(self, value):
        return {"fieldName": self.name, "operator": "regexp", "value": str(value)}

    def __str__(self):
        return self.name



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log

'''AppCategoryTestCase tests the app category object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for AppCategory TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for AppCategory TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_app_category_details(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().application_categories.list()
    assert resp is not None
    # atleast 10 default app category are there
    assert resp.__len__() >= 10



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class FibreChannelInitiatorAlias(Resource):
    """This API provides the alias information for Fibre Channel initiators.

    # Parameters
    id     : Unique identifier for the Fibre Channel initiator alias.
    alias  : Alias of the Fibre Channel initiator.
    wwpn   : WWPN (World Wide Port Name) of the Fibre Channel initiator.
    source : Source of the Fibre Channel initiator alias.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class FibreChannelInitiatorAliasList(Collection):
    resource = FibreChannelInitiatorAlias
    resource_type = "fibre_channel_initiator_aliases"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions


'''ShelveTestCase class tests the subnet object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Shelves TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Shelves TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_shelve(setup_teardown_for_each_test):
    # sdk bug. why is subnet object having functions like create,
    # update delete??? the rest doc does not have these. only read is allowed
    resp = nimosclientbase.get_nimos_client().shelves.list(detail=True)
    assert resp is not None
    # doc shows it has 13 properties.just check the length
    assert resp[0].attrs.__len__() == 13


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_activateshelve(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().shelves.get()
    assert resp is not None
    try:
        resp = nimosclientbase.get_nimos_client().shelves.update(
                id=resp.attrs.get("id"), force=True, activated=True)
        assert resp.attrs.get("activated") is True
    except exceptions.NimOSAPIError as ex:
        if "SM_shelf_no_eloc_id" in str(ex):
            log("making test as passed since no shelve exist for expansion")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex
    assert resp is not None



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class KeyManager(Resource):
    """Key Manager stores encryption keys for the array volumes / dedupe domains.

    # Parameters
    id          : Identifier for External Key Manager.
    name        : Name of external key manager.
    description : Description of external key manager.
    hostname    : Hostname or IP Address for the External Key Manager.
    port        : Port number for the External Key Manager.
    protocol    : KMIP protocol supported by External Key Manager.
    username    : External Key Manager username. String up to 255 printable characters.
    password    : External Key Manager user password. String up to 255 printable characters.
    active      : Whether the given key manager is active or not.
    status      : Connection status of a given external key manager.
    vendor      : KMIP vendor name.
    """
    def remove(self, **kwargs):
        """Remove external key manager. You must migrate the keys to an inactive external key manager before removing the active key manager. If you remove the active external key
        manager the passphrase is used to enable the internal key manager.

        # Parameters
        id         : ID of the external key manager.
        passphrase : Passphrase used to protect the master key, required during deletion of external key manager.
        """

        return self._collection.remove(
            self.id,
            **kwargs
        )
    def migrate_keys(self, **kwargs):
        """Migrate volume encryption keys from the active key manager to the destination id given in the input. After successfully migrating the encryption keys, the destination key
        manager is made the active key manager.

        # Parameters
        id : ID of the destination external key manager.
        """

        return self._collection.migrate_keys(
            self.id,
            **kwargs
        )

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")


class KeyManagerList(Collection):
    resource = KeyManager
    resource_type = "key_managers"

    def remove(self, id, **kwargs):
        """Remove external key manager. You must migrate the keys to an inactive external key manager before removing the active key manager. If you remove the active external key
        manager the passphrase is used to enable the internal key manager.

        # Parameters
        id         : ID of the external key manager.
        passphrase : Passphrase used to protect the master key, required during deletion of external key manager.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'remove',
            id=id,
            **kwargs
        )

    def migrate_keys(self, id, **kwargs):
        """Migrate volume encryption keys from the active key manager to the destination id given in the input. After successfully migrating the encryption keys, the destination key
        manager is made the active key manager.

        # Parameters
        id : ID of the destination external key manager.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'migrate_keys',
            id=id,
            **kwargs
        )

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

# global variables
key_name_1 = "default"
passphrase = "passphrase-91"
master_key_to_delete = []

'''master_keyTestCase tests the master key object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for master_key TestCase *****\n")
    resp = nimosclientbase.get_nimos_client().master_key.get()
    if resp is not None:
        nimosclientbase.get_nimos_client().master_key.delete(resp.attrs.get("id"))

    def after_running_all_testcase():
        log("**** Completed Tests for master_key TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_master_key()
    nimosclientbase.log_footer(request.function.__name__)


def delete_master_key():
    for master_key_id in master_key_to_delete:
        nimosclientbase.get_nimos_client().master_key.delete(master_key_id)
        log(f"Deleted master key with Id '{master_key_id}'")
    master_key_to_delete.clear()


def create_master_key(master_key, **kwargs):
    resp = nimosclientbase.get_nimos_client().master_key.create(
        name=master_key, **kwargs)
    master_key_id = resp.attrs.get("id")
    master_key_to_delete.append(master_key_id)
    assert resp is not None
    log("Created master key with name "
        f"'{master_key}' and Id '{master_key_id}'")
    return resp


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_master_key(setup_teardown_for_each_test):
    resp = create_master_key(key_name_1,
                             passphrase=passphrase)
    resp = nimosclientbase.get_nimos_client().master_key.list(
        detail=True, limit=2)
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_create_master_key(setup_teardown_for_each_test):
    try:
        resp = create_master_key(key_name_1,
                                 passphrase=passphrase)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_check_mandatory_params_master_key(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().master_key.create(
            name=key_name_1)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log("Failed as expected. missing mandatory arguments.")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_master_key_endrow_beyond(setup_teardown_for_each_test):
    try:
        resp = create_master_key(key_name_1,
                                 passphrase=passphrase)
        resp = nimosclientbase.get_nimos_client().master_key.get(endRow=30)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected.no rows present")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex

@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_select_fields_for_master_key(setup_teardown_for_each_test):
    try:
        resp = create_master_key(key_name_1,
                                 passphrase=passphrase)
        resp = nimosclientbase.get_nimos_client().master_key.get(
            fields="name,id,active")
        assert resp is not None
        assert resp.attrs.get("name") is not None
        assert resp.attrs.get("id") is not None
        assert resp.attrs.get("active") is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_master_key(setup_teardown_for_each_test):
    try:
        resp = create_master_key(key_name_1,
                                 passphrase=passphrase)
        assert resp is not None
        # update few fields
        update_resp = nimosclientbase.get_nimos_client().master_key.update(
            id=resp.attrs.get("id"),
            name=key_name_1,
            passphrase=passphrase,
            active=False)
        assert update_resp is not None
        # assert the values got updated
        assert update_resp.attrs.get("name") == key_name_1
        assert update_resp.attrs.get("active") is False
        assert update_resp.attrs.get("id") == resp.attrs.get("id")
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

# global variables
perf_policy_name1 = ""
performancepolicy_to_delete = []

'''PerformancePoliciesTestCase tests the Performance
     Policies object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Performance Policies TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Performance Policies TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    global perf_policy_name1
    # setup operations before yield is called
    perf_policy_name1 = nimosclientbase.get_unique_string(
        "unittestcase-perfpolicy1")
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_perf_policy()
    nimosclientbase.log_footer(request.function.__name__)


def delete_perf_policy():
    for perf_policy_id in performancepolicy_to_delete:
        try:
            resp = nimosclientbase.get_nimos_client(
            ).performance_policies.delete(id=perf_policy_id)
            assert resp is not None
            log("Deleted performance policy with Id '{perf_policy_id}'")
        except exceptions.NimOSAPIError as ex:
            log(f"Failed with exception message : {str(ex)}")
            raise ex
    performancepolicy_to_delete.clear()


def create_perf_policy(perf_policy_name, **kwargs):
    resp = nimosclientbase.get_nimos_client().performance_policies.create(
        name=perf_policy_name, **kwargs)
    perf_policy_id = resp.attrs.get("id")
    performancepolicy_to_delete.append(perf_policy_id)
    assert resp is not None
    log("Created performance policy with name "
        f"'{perf_policy_name}' and Id '{perf_policy_id}'")
    assert resp is not None
    return resp


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_perf_policies(setup_teardown_for_each_test):
    # only . and : and - are allowed as esp characte..
    try:
        # first get the default ones
        getall_resp = nimosclientbase.get_nimos_client(
        ).performance_policies.list()
        assert getall_resp is not None
        # create one
        resp = create_perf_policy(
            perf_policy_name1,
            description="created by testcase",
            block_size=8192,
        )
        assert resp is not None
        # match the value
        get_resp = nimosclientbase.get_nimos_client().performance_policies.get(
            id=resp.attrs.get("id"))
        assert get_resp is not None
        assert get_resp.attrs.get("id") == performancepolicy_to_delete[0]
        assert get_resp.attrs.get("description") == "created by testcase"
        assert get_resp.attrs.get("block_size") == 8192
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_perf_policies_using_invalid_app_category(
        setup_teardown_for_each_test):
    try:
        # create one
        resp = create_perf_policy(
            perf_policy_name1,
            description="created by testcase",
            block_size=8192,
            app_category="test"  # invalid app category
        )
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_perfpol_invalid_app_category" in str(ex):
            log("Failed as expected. Invalid app category provided.")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_perf_policies_using_invalid_name(setup_teardown_for_each_test):
    try:
        # create one
        perf_name = perf_policy_name1 + "-;,"
        resp = create_perf_policy(
            perf_name,
            description="created by testcase",
            block_size=8192,
            app_category="test"  # invalid app category
        )
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. Invalid name provided {perf_name}")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_perf_policies(setup_teardown_for_each_test):
    try:
        # create one
        resp = create_perf_policy(
            perf_policy_name1,
            description="created by testcase",
            block_size=8192,
            app_category="db2"
        )
        assert resp is not None
        # match the value
        get_resp = nimosclientbase.get_nimos_client(
        ).performance_policies.get(id=resp.attrs.get("id"))
        assert get_resp is not None
        assert get_resp.attrs.get("id") == performancepolicy_to_delete[0]
        assert get_resp.attrs.get("description") == "created by testcase"
        assert get_resp.attrs.get("block_size") == 8192
        assert get_resp.attrs.get("app_category") == "db2"

        # update the app category field
        update_resp = nimosclientbase.get_nimos_client(
        ).performance_policies.update(id=resp.attrs.get("id"),
                                      description="modified by testcase",
                                      app_category="exchange")
        assert update_resp is not None
        assert update_resp .attrs.get("id") == performancepolicy_to_delete[0]
        assert update_resp .attrs.get("description") == "modified by testcase"
        assert update_resp .attrs.get("app_category") == "exchange"

    except exceptions.NimOSAPIError as ex:
        if "SM_perfpol_invalid_app_category" in str(ex):
            log("Failed as expected. Invalid app category provided.")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_delete_perf_policies(setup_teardown_for_each_test):
    try:
        # create one
        resp = create_perf_policy(perf_policy_name1,
                                  description="created by testcase",
                                  block_size=8192,
                                  app_category="db2"
                                  )
        assert resp is not None
        # match the value
        get_resp = nimosclientbase.get_nimos_client(
            ).performance_policies.get(id=resp.attrs.get("id"))
        assert get_resp is not None
        assert get_resp.attrs.get("id") == performancepolicy_to_delete[0]
        assert get_resp.attrs.get("description") == "created by testcase"
        assert get_resp.attrs.get("block_size") == 8192
        assert get_resp.attrs.get("app_category") == "db2"

        # delete the policy
        delete_resp = nimosclientbase.get_nimos_client(
                ).performance_policies.delete(id=get_resp.attrs.get("id"))
        assert delete_resp . __len__() == 0
        performancepolicy_to_delete.remove(resp.attrs.get("id"))
    except exceptions.NimOSAPIError as ex:
        if "SM_perfpol_invalid_app_category" in str(ex):
            log("Failed as expected. Invalid app category provided.")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

# global variables
chap_name_1 = nimosclientbase.get_unique_string("chapusertc-1")
chap_password = "password_25-24"
chap_user_to_delete = []

'''Chapusers TestCase tests the chap users object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for chap_users TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for chap_users TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_chap_user()
    nimosclientbase.log_footer(request.function.__name__)


def delete_chap_user():
    for chap_user_id in chap_user_to_delete:
        nimosclientbase.get_nimos_client().chap_users.delete(chap_user_id)
        log(f" Deleted chap user with id '{chap_user_id}'")
    chap_user_to_delete.clear()


def create_chap_user(user_name, password, **kwargs):
    resp = nimosclientbase.get_nimos_client().chap_users.create(
        name=user_name, password=password, **kwargs)
    chap_user_id = resp.attrs.get("id")
    chap_user_to_delete.append(chap_user_id)
    assert resp is not None
    log(f"Created chap user with name '{user_name} ' and Id '{chap_user_id}'")
    return resp


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_chap_users(setup_teardown_for_each_test):
    resp = create_chap_user(
        user_name=chap_name_1,
        password=chap_password,
        description="created by testcase"
    )
    resp = nimosclientbase.get_nimos_client().chap_users.list(
        detail=True, limit=2)
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_chap_users(setup_teardown_for_each_test):
    resp = create_chap_user(
        user_name=chap_name_1,
        password=chap_password,
        description="created by testcase"
    )
    assert resp is not None
    assert resp.attrs.get("name") == chap_name_1
    assert resp.attrs.get("description") == "created by testcase"


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_chap_users_using_invalid_password(setup_teardown_for_each_test):
    try:
        resp = create_chap_user(
            user_name=chap_name_1,
            password="sadhs",
            description="created by testcase"
        )
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log("Failed as expected. password length short")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_delete_invalid_chap_users(setup_teardown_for_each_test):
    resp = create_chap_user(
        user_name=chap_name_1,
        password=chap_password,
        description="created by testcase"
    )
    assert resp is not None
    assert resp.attrs.get("name") == chap_name_1
    assert resp.attrs.get("description") == "created by testcase"
    try:
        resp = nimosclientbase.get_nimos_client().chap_users.delete(
            id="213812497124712041adhjasjdgassqqwjahsdaskdk")
    except exceptions.NimOSAPIError as ex:
        if"SM_invalid_path_variable" in str(ex):
            log("Failed as expected. Invalid Id to delete")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_chap_users(setup_teardown_for_each_test):
    resp = create_chap_user(
        user_name=chap_name_1,
        password=chap_password,
        description="created by testcase"
    )
    assert resp is not None
    assert resp.attrs.get("name") == chap_name_1
    assert resp.attrs.get("description") == "created by testcase"
    # update few fields
    update_resp = nimosclientbase.get_nimos_client().chap_users.update(
        id=resp.attrs.get("id"),
        description="modified by testcase",
        name="updatechapusertestcase")
    update_resp is not None
    assert update_resp.attrs.get("name") == "updatechapusertestcase"
    assert update_resp.attrs.get("description") == "modified by testcase"



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class Folder(Resource):
    """Folders are a way to group volumes, as well as a way to apply space constraints to them.

    # Parameters
    id                            : Identifier for the folder.
    name                          : Name of folder.
    fqn                           : Fully qualified name of folder in the pool.
    full_name                     : Fully qualified name of folder in the group.
    search_name                   : Name of folder used for object search.
    description                   : Text description of folder.
    pool_name                     : Name of the pool where the folder resides.
    pool_id                       : ID of the pool where the folder resides.
    limit_bytes_specified         : Indicates whether the folder has a limit.
    limit_bytes                   : Folder limit size in bytes. By default, a folder (except SMIS and VVol types) does not have a limit. If limit_bytes is not specified when a
                                    folder is created, or if limit_bytes is set to the largest possible 64-bit signed integer (9223372036854775807), then the folder has no limit.
                                    Otherwise, a limit smaller than the capacity of the pool can be set. On output, if the folder has a limit, the limit_bytes_specified attribute
                                    will be true and limit_bytes will be the limit. If the folder does not have a limit, the limit_bytes_specified attribute will be false and
                                    limit_bytes will be interpreted based on the value of the usage_valid attribute. If the usage_valid attribute is true, limits_byte will be the
                                    capacity of the pool. Otherwise, limits_bytes is not meaningful and can be null. SMIS and VVol folders require a size limit. This attribute is
                                    superseded by limit_size_bytes.
    limit_size_bytes              : Folder size limit in bytes. If limit_size_bytes is not specified when a folder is created, or if limit_size_bytes is set to -1, then the folder
                                    has no limit. Otherwise, a limit smaller than the capacity of the pool can be set. Folders with an agent_type of 'smis' or 'vvol' must have a
                                    size limit.
    provisioned_limit_size_bytes  : Limit on the provisioned size of volumes in a folder. If provisioned_limit_size_bytes is not specified when a folder is created, or if
                                    provisioned_limit_size_bytes is set to -1, then the folder has no provisioned size limit.
    overdraft_limit_pct           : Amount of space to consider as overdraft range for this folder as a percentage of folder used limit. Valid values are from 0% - 200%. This is
                                    the limit above the folder usage limit beyond which enforcement action(volume offline/non-writable) is issued.
    capacity_bytes                : Capacity of the folder in bytes. If the folder's size has a usage limit, capacity_bytes will be the folder's usage limit. If the folder's size
                                    does not have a usage limit, capacity_bytes will be the pool's capacity. This field is meaningful only when the usage_valid attribute is true.
    free_space_bytes              : Free space in the folder in bytes. If the folder has a usage limit, free_space_bytes will be the folder's free space (the folder's usage limit
                                    minus the folder's space usage). If the folder does not have a usage limit, free_space_bytes will be the pool's free space. This field is
                                    meaningful only when the usage_valid attribute is true.
    provisioned_bytes             : Sum of provisioned size of volumes in the folder.
    usage_bytes                   : Sum of mapped usage and snapshot uncompressed usage of volumes in the folder.
    volume_mapped_bytes           : Sum of mapped usage of volumes in the folder.
    usage_valid                   : Indicate whether the space usage attributes of folder are valid.
    agent_type                    : External management agent type.
    inherited_vol_perfpol_id      : Identifier of the default performance policy for a newly created volume.
    inherited_vol_perfpol_name    : Name of the default performance policy for a newly created volume.
    unused_reserve_bytes          : Unused reserve of volumes in the folder in bytes. This field is meaningful only when the usage_valid attribute is true.
    unused_snap_reserve_bytes     : Unused reserve of snapshots of volumes in the folder in bytes. This field is meaningful only when the usage_valid attribute is true.
    compressed_vol_usage_bytes    : Compressed usage of volumes in the folder. This field is meaningful only when the usage_valid attribute is true.
    compressed_snap_usage_bytes   : Compressed usage of snapshots in the folder. This field is meaningful only when the usage_valid attribute is true.
    uncompressed_vol_usage_bytes  : Uncompressed usage of volumes in the folder. This field is meaningful only when the usage_valid attribute is true.
    uncompressed_snap_usage_bytes : Uncompressed usage of snapshots in the folder. This field is meaningful only when the usage_valid attribute is true.
    vol_compression_ratio         : Compression ratio of volumes in the folder. This field is meaningful only when the usage_valid attribute is true.
    snap_compression_ratio        : Compression ratio of snapshots in the folder. This field is meaningful only when the usage_valid attribute is true.
    compression_ratio             : Compression savings for the folder expressed as ratio. This field is meaningful only when the usage_valid attribute is true.
    creation_time                 : Time when this folder was created.
    last_modified                 : Time when this folder was last modified.
    num_snaps                     : Number of snapshots inside the folder. This attribute is deprecated and has no meaningful value.
    num_snapcolls                 : Number of snapshot collections inside the folder. This attribute is deprecated and has no meaningful value.
    app_uuid                      : Application identifier of the folder.
    volume_list                   : List of volumes contained by the folder.
    appserver_id                  : Identifier of the application server associated with the folder.
    appserver_name                : Name of the application server associated with the folder.
    folset_id                     : Identifier of the folder set associated with the folder. Only VVol folder can be associated with the folder set. The folder and the containing
                                    folder set must be associated with the same application server.
    folset_name                   : Name of the folder set associated with the folder. Only VVol folder can be associated with the folder set. The folder and the containing folder
                                    set must be associated with the same application server.
    limit_iops                    : IOPS limit for this folder. If limit_iops is not specified when a folder is created, or if limit_iops is set to -1, then the folder has no IOPS
                                    limit. IOPS limit should be in range [256, 4294967294] or -1 for unlimited.
    limit_mbps                    : Throughput limit for this folder in MB/s. If limit_mbps is not specified when a folder is created, or if limit_mbps is set to -1, then the
                                    folder has no throughput limit. MBPS limit should be in range [1, 4294967294] or -1 for unlimited.
    access_protocol               : Access protocol of the folder. This attribute is used by the VASA Provider to determine the access protocol of the bind request. If not
                                    specified in the creation request, it will be the access protocol supported by the group. If the group supports multiple protocols, the default
                                    will be Fibre Channel. This field is meaningful only to VVol folder.
    tenant_id                     : Tenant ID of the folder. This is used to determine what tenant context the folder belongs to.
    """
    def set_dedupe(self, dedupe_enabled, **kwargs):
        """Set dedupe enabled/disabled for all applicable volumes inside a folder.

        # Parameters
        dedupe_enabled : Enable/disable dedupe.
        id             : Folder containing the volumes to enable/disable dedupe on.
        """

        return self._collection.set_dedupe(
            self.id,
            dedupe_enabled,
            **kwargs
        )


class FolderList(Collection):
    resource = Folder
    resource_type = "folders"

    def set_dedupe(self, id, dedupe_enabled, **kwargs):
        """Set dedupe enabled/disabled for all applicable volumes inside a folder.

        # Parameters
        dedupe_enabled : Enable/disable dedupe.
        id             : Folder containing the volumes to enable/disable dedupe on.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'set_dedupe',
            dedupe_enabled=dedupe_enabled,
            id=id,
            **kwargs
        )



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class FibreChannelInterface(Resource):
    """Represent information of specified Fibre Channel interfaces. Fibre Channel interfaces are hosted on Fibre Channel ports to provide data access.

    # Parameters
    id                   : Identifier for the Fibre Channel interface.
    array_name_or_serial : Name or serial number of array where the interface is hosted.
    partial_response_ok  : Indicate that it is ok to provide partially available response.
    controller_name      : Name (A or B) of the controller where the interface is hosted.
    fc_port_id           : ID of the port with which the interface is associated.
    name                 : Name of Fibre Channel interface.
    wwnn                 : WWNN (World Wide Node Name) for this Fibre Channel interface.
    wwpn                 : WWPN (World Wide Port Name) for this Fibre Channel interface.
    peerzone             : Active peer zone for this Fibre Channel interface.
    online               : Identify whether the Fibre Channel interface is online.
    firmware_version     : Version of the Fibre Channel firmware.
    logical_port_number  : Logical port number for the Fibre Channel port.
    fc_port_name         : Name of Fibre Channel port.
    bus_location         : PCI bus location of the HBA for this Fibre Channel port.
    slot                 : HBA slot number for this Fibre Channel port.
    orientation          : Orientation of FC ports on a HBA. An orientation of 'right_to_left' indicates that ports are ordered as 3,2,1,0 on the slot. Possible values:
                           'left_to_right', 'right_to_left'.
    port                 : HBA port number for this Fibre Channel port.
    link_info            : Information about the Fibre Channel link at which interface is operating.
    fabric_info          : Fibre Channel fabric information.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")


class FibreChannelInterfaceList(Collection):
    resource = FibreChannelInterface
    resource_type = "fibre_channel_interfaces"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Disk(Resource):
    """Disks are used for storing user data.

    # Parameters
    id                        : ID of disk.
    is_dfc                    : Is disk part of dual flash carrier.
    serial                    : Disk serial number(N/A if empty).
    path                      : Disk SCSI device path.
    shelf_serial              : Serial number of the shelf the disk is attached to.
    shelf_location            : Identifies the controller, port, and chain position of the shelf the disk belongs to.
    shelf_id                  : Identifies the physical shelf the disk belongs to.
    shelf_location_id         : Identifies the position shelf the disk belongs to, as coded integer.
    vshelf_id                 : Identifies the local shelf ID the disk belongs to.
    slot                      : Disk slot number.
    bank                      : Disk bank number.
    model                     : Disk model name.
    vendor                    : Vendor name of the disk manufacturer.
    firmware_version          : Firmware version on the disk.
    hba                       : HBA ID the disk is connected to.
    port                      : HBA port number the disk is connected to.
    size                      : Disk size in bytes.
    state                     : Disk hardware state.
    type                      : Type of disk (HDD, SSD, N/A).
    block_type                : Native block type of the disk.
    raid_id                   : Raid ID.
    raid_resync_percent       : Percentage RAID rebuild completed on this disk.
    raid_resync_current_speed : Current RAID rebuild speed (bytes/sec).
    raid_resync_average_speed : Average RAID rebuild speed (bytes/sec).
    raid_state                : RAID status for the disk (N/A, okay, resynchronizing, spare, faulty).
    disk_internal_stat_1      : Internal disk statistic 1.
    smart_attribute_list      : S.M.A.R.T. attributes for the disk.
    disk_op                   : The intended operation to be performed on the specified disk.
    force                     : Forcibly add a disk.
    array_name                : Name of array the disk belongs to.
    array_id                  : ID of array the disk belongs to.
    partial_response_ok       : Indicate that it is okay to provide partially available response.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")


class DiskList(Collection):
    resource = Disk
    resource_type = "disks"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Event(Resource):
    """View events.

    # Parameters
    id          : Identifier for the event record.
    type        : Type of the event record.
    name        : Name of alert macro to generate.
    scope       : The array name for array level event.
    target      : Name of object upon which the event occurred.
    target_type : Target type of the event record.
    timestamp   : Time when this event happened.
    category    : Category of the event record.
    severity    : Severity level of the event.
    summary     : Summary of the event.
    activity    : Description of the event.
    alarm_id    : The alarm ID if the event is related to an alarm.
    params      : Arguments provided for event creation in key-value structure. Until KV implementation for events, will ignore keys (though keys must be non-empty) and take
                  values positionally.
    tenant_id   : Tenant ID of the event. This is used to determine what tenant context the event belongs to.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class EventList(Collection):
    resource = Event
    resource_type = "events"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")
